
from sympy.core.add import Add
from sympy.core.mul import Mul
from sympy.core.power import Pow
from sympy.core.sympify import sympify
from sympy.core.basic import Basic, S, C
from sympy.core.symbol import Symbol, Wild
from sympy.core.numbers import Integer, igcd, ilcm

from sympy.utilities import all, any

from sympy.polys.monomial import monomial_cmp, monomial_mul, \
    monomial_div, monomial_max, monomial_min, monomial_as_basic

import sympy.polys

import math

class SymbolsError(Exception):
    pass

class PolynomialError(Exception):
    pass

class CoefficientError(PolynomialError):
    pass

class UnivariatePolyError(PolynomialError):

    def __init__(self, poly):
        self.message = "%s is an univariate polynomial" % poly

    def __str__(self):
        return self.message

class MultivariatePolyError(PolynomialError):

    def __init__(self, poly):
        self.message = "%s is a multivariate polynomial" % poly

    def __str__(self):
        return self.message

##
## TODO:
##
##  [!] Copy + Write tests for everything.
##  [2] Analyze dense MV multiplication.
##  [~] Create MV poly with int coeffs.
##  [4] Think on GF polys (for factoring).
##  [5] Improve coefficients analysis.
##  [6] Implement FactoredPoly.
##  [7] Concept of Monomial.
##

class Poly(Basic):
    """Represents polynomials with symbolic coefficients.

       Polynomials are internally represented as two lists containing
       coefficients and monomials (tuples of exponents) respectively.
       Stored are only terms with non-zero coefficients, so generally
       all polynomials are considered sparse. However algorithms will
       detect dense polynomials and use the appropriate method to
       solve the given problem in the most efficient way.

       The most common way to initialize a polynomial instance is to
       provide a valid expression together witch a set of ordered
       symbols and, additionally, monomial ordering:

            Poly(expression, x_1, x_2, ..., x_n, order='grlex')

       If the given expression is not a polynomial with respect to
       the given variables, an exception is raised. Alternatively
       you can use Basic.as_poly to avoid exception handling.

       By default ordering of monomials can be omitted. In this case
       graded lexicographic order will be used. Anyway remember that
       'order' is a keyword argument.

       Currently there are supported four standard orderings:

           [1] lex       -> lexicographic order
           [2] grlex     -> graded lex order
           [3] grevlex   -> reversed grlex order
           [4] 1-el      -> first elimination order

       Polynomial can be also constructed explicitly by specifying
       a collection of coefficients and monomials. This can be done
       in at least three different ways:

           [1] [(c_1, M_1), (c_2, M_2), ..., (c_1, M_1)]

           [2] (c_1, c2, ..., c_n), (M_1, M_2, ..., M_n)

           [3] { M_1 : c_1, M_2 : c_2, ..., M_n : c_n }

       Although all three representation look similar, they are
       designed for different tasks and have specific properties:

           [1] All coefficients and monomials  are validated before
               polynomial instance is created. Monomials are sorted
               with respect to the given order.

           [2] No validity checking, no sorting, just a raw copy.

           [3] Also no validity checking however monomials are
               sorted with respect to the given order.

       For interactive usage choose [1] as it's safe and relatively
       fast. Use [2] or [3] internally for time critical algorithms,
       when you know that coefficients and monomials will be valid.

       Implemented methods:

           U --> handles uniformly int | long and Basic coefficients
                 (other methods need to be overridden in IntegerPoly)

           P --> a property (with appropriate decorator)

           - --> otherwise

       [1] Representation conversion:

          [1.1] [--] as_basic   --> converts to a valid sympy expression
          [1.2] [U-] as_dict    --> returns { M_1 : c_1, ..., M_1 : c_1 }
          [1.3] [U-] as_uv_dict --> returns dict with integer keys

       [2] Polynomial internals:

          [2.1] [UP] coeffs  --> list of coefficients
          [2.2] [UP] monoms  --> list of monomials
          [2.3] [UP] symbols --> an ordered list of symbols
          [2.4] [UP] order   --> the ordering of monomials
          [2.5] [UP] stamp   --> frozen set of polynomial's variables
          [2.6] [UP] domain  --> coefficient domain, see [8] for details
          [2.7] [UP] args    --> required for printing and hashing
          [2.8] [UP] flags   --> dict of keyword arguments to Poly

       [3] General characteristics:

          [3.1] [UP] norm         --> computes oo-norm of coefficients
          [3.2] [UP] length       --> counts the total number of terms
          [3.3] [UP] degree       --> returns degree of the leading monomial
          [3.4] [UP] total_degree --> returns true total degree of a polynomial

       [4] Items generators:

          [4.1] [U-] iter_coeffs     --> iterate over coefficients
          [4.2] [U-] iter_monoms     --> iterate over monomials
          [4.3] [U-] iter_terms      --> iterate over terms

          [4.4] [U-] iter_all_coeffs --> iterate over all coefficients
          [4.5] [U-] iter_all_monoms --> iterate over all monomials
          [4.6] [U-] iter_all_terms  --> iterate over all terms

       [5] Leading and trailing items:

          [5.1] [UP] lead_coeff or LC --> returns the leading coefficient
          [5.2] [UP] lead_monom or LM --> returns the leading monomial
          [5.3] [UP] lead_term  or LT --> returns the leading term

          [5.4] [UP] tail_coeff or TC --> returns the trailing coefficient
          [5.5] [UP] tail_monom or TM --> returns the trailing monomial
          [5.6] [UP] tail_term  or TT --> returns the trailing term

       [6] Arithmetic operators:

          [6.1]  [U-] __abs__     --> for all i, abs(c_i)
          [6.2]  [U-] __neg__     --> polynomial negation
          [6.3]  [U-] __add__     --> polynomial addition
          [6.4]  [U-] __sub__     --> polynomial subtraction
          [6.5]  [--] __mul__     --> polynomial multiplication
          [6.6]  [U-] __pow__     --> polynomial exponentiation
          [6.7]  [U-] __div__     --> polynomial quotient
          [6.8]  [U-] __mod__     --> polynomial remainder
          [6.9]  [U-] __divmod__  --> both 'div' and 'mod'
          [6.10] [U-] __truediv__ --> the same as 'div'

       [7] Polynomial properties:

          [7.1]  [UP] is_zero          --> only one term c_0 == 0
          [7.2]  [UP] is_one           --> only one term c_0 == 1
          [7.3]  [-P] is_number        --> only numeric constant term
          [7.4]  [UP] is_constant      --> only arbitrary constant term
          [7.5]  [UP] is_monomial      --> number of terms == 1
          [7.6]  [UP] is_univariate    --> number of variables == 1
          [7.7]  [UP] is_multivariate  --> number of variables > 1
          [7.8]  [UP] is_homogeneous   --> has constant term
          [7.9]  [UP] is_inhomogeneous --> no constant term
          [7.10] [UP] is_sparse        --> filled with less than 90% of terms
          [7.11] [UP] is_dense         --> filled with more than 90% of terms
          [7.12] [UP] is_monic         --> returns True if leading coeff is one
          [7.13] [UP] is_primitive     --> returns True if content is one
          [7.14] [UP] is_squarefree    --> no factors of multiplicity >= 2
          [7.15] [UP] is_linear        --> all monomials of degree <= 1

       [8] Coefficients properties:

          [8.1] [UP] has_integer_coeffs  --> for all i, c_i in Z
          [8.2] [UP] has_rational_coeffs --> for all i, c_i in Q
          [8.3] [UP] has_radical_coeffs  --> for all i, c_i in R
          [8.4] [UP] has_complex_coeffs  --> for all i, c_i in C
          [8.5] [UP] has_symbolic_coeffs --> otherwise

       [9] Special functionality:

          [9.1]  [--] as_monic      --> divides all coefficients by LC
          [9.2]  [--] as_integer    --> returns polynomial with integer coeffs

          [9.3]  [-P] content       --> returns GCD of polynomial coefficients
          [9.4]  [--] as_primitive  --> returns content and primitive part

          [9.5]  [U-] as_squarefree --> returns square-free part of a polynomial

          [9.6]  [U-] as_reduced    --> extract GCD of polynomial monomials

          [9.7]  [--] map_coeffs    --> applies a function to all coefficients
          [9.8]  [U-] coeff         --> returns coefficient of the given monomial

          [9.9]  [U-] unify_with    --> returns polys with a common set of symbols

          [9.10] [U-] diff          --> efficient polynomial differentiation
          [9.11] [U-] integrate     --> efficient polynomial integration

          [9.12] [U-] invert        --> computes multiplicative inverse in K[t]

       [10] Operations on terms:

          [10.1] [U-] add_term --> add a term to a polynomial
          [10.2] [U-] sub_term --> subtract a term from a polynomial

          [10.3] [--] mul_term --> multiply a polynomial with a term
          [10.4] [--] div_term --> divide a polynomial by a term

          [10.5] [U-] remove_lead_term --> remove leading term (up to order)
          [10.6] [U-] remove_last_term --> remove last term (up to order)

       [11] Substitution and evaluation:

          [11.1] [U-] __call__    --> evaluates poly a the given point
          [11.2] [U-] evaluate    --> evaluates poly for specific vars
          [11.3] [--] _eval_subs  --> efficiently substitute variables

       [12] Comparison functions:

          [12.1] [U-] __eq__      --> P1 == P, up to order of monomials
          [12.2] [U-] __ne__      --> P1 != P, up to order of monomials
          [12.3] [U-] __nonzero__ --> check if zero polynomial

       [13] String representation functions:

          [13.1] [U-] repr      --> returns represantation of 'self'
          [13.3] [U-] str       --> returns pretty representation

       [14] Other (derived from Basic) functionality:

          [14.1] [U-] _eval_is_polynomial --> checks if poly is a poly

       For general information on polynomials and algorithms, refer to:

       [1] D. E. Knuth, The Art of Computer Programming: Seminumerical
           Algorithms, v.2, Addison-Wesley Professional, 1997

       [2] J. von zur Gathen, J. Gerhard, Modern Computer Algebra,
           Second Edition, Cambridge University Press, 2003

       [3] K. Geddes, S. R. Czapor, G. Labahn, Algorithms for
           Computer Algebra, Springer, 1992

       [4] D. Bini, V. Y.Pan, Polynomial and Matrix Computations:
           Volume 1: Fundamental Algorithms, Birkhauser Boston, 1994

       [5] R. T. Moenck, Practical Fast Polynomial Multiplication,
           Proceedings of the third ACM symposium on Symbolic and
           algebraic computation, ACM Press, Yorktown Heights,
           New York, USA, 1976, pp. 136-148

       [6] M. Bronstein, Symbolic Integration I: Transcendental
           Functions, Second Edition, Springer-Verlang, 2005

       See also documentation of particular methods.

    """

    def __new__(cls, poly, *symbols, **flags):
        N, terms = len(symbols), {}
        coeffs, monoms = (), ()

        order = flags.pop('order', 'grlex')

        if N == 0:
            if isinstance(poly, Poly):
                if order != poly.order:
                    symbols = poly.symbols
                    N = len(poly.symbols)
                    poly = poly.as_dict()
                else:
                    return poly
            else:
                raise SymbolsError("No symbols were given")

        stamp = frozenset(symbols)

        if len(stamp) != N:
            raise SymbolsError("Duplicate symbols: %s" % (symbols,))

        symbols = tuple(map(sympify, symbols))

        if any(not s.is_Symbol for s in symbols):
            raise SymbolsError("Invalid symbols: %s" % (symbols,))

        if any(not s.is_commutative for s in symbols):
            raise SymbolsError("Non-commutative symbols: %s" % (symbols,))

        # { M1: c1, M2: c2, ... }
        if type(poly) is dict:
            if not poly:
                coeffs = (S.Zero,)
                monoms = ((0,) * N,)
            else:
                terms = poly

        # ((c1, c2, ...), (M1, M2, ...))
        elif type(poly) is tuple:
            if not poly:
                coeffs = (S.Zero,)
                monoms = ((0,) * N,)
            else:
                if len(poly) == 2:
                    coeffs, monoms = poly[0], poly[1]

                    if isinstance(coeffs, (tuple, list)):
                        if not (coeffs or monoms):
                            coeffs = (S.Zero,)
                            monoms = ((0,) * N,)
                    else:
                        coeffs = (coeffs,)
                        monoms = (monoms,)
                else:
                    raise PolynomialError("Invalid arguments," \
                        " should get '(coeffs, monoms)' tuple")

        # [ (c1,M1), (c2,M2), ... ]
        elif type(poly) is list:
            if not poly:
                coeffs = (S.Zero,)
                monoms = ((0,) * N,)
            else:
                for coeff, monom in poly:
                    coeff = sympify(coeff)

                    if coeff.has_any_symbols(*symbols):
                        raise CoefficientError("%s coefficient is dependent" \
                            " of polynomial's symbols %s" % (coeff, symbols))

                    if type(monom) is int:
                        monom = (monom,)

                    if len(monom) != N:
                        raise PolynomialError("Dimension of %s monomial does" \
                            " not match the number of symbols (%d)" % (monom, N))

                    if not coeff:
                        continue

                    if terms.has_key(monom):
                        coeff += terms[monom]

                        if not coeff:
                            del terms[monom]
                            continue

                    terms[monom] = coeff

                if not terms:
                    coeffs = (S.Zero,)
                    monoms = ((0,) * N,)
        elif isinstance(poly, Poly):
            if stamp <= poly.stamp:
                if symbols == poly.symbols:
                    if N > 1 and order != poly.order:
                        terms = poly.as_dict()
                    else:
                        return poly
                else:
                    terms = Poly._permute(poly, *symbols)
            else:
                if any(coeff.has_any_symbols(*symbols) for coeff in poly.coeffs):
                    terms = Poly._decompose(poly.as_basic(), *symbols)
                else:
                    K = len(poly.symbols)

                    if symbols[:K] == poly.symbols:
                        coeffs, T = poly.coeffs, (0,) * (N - K)
                        monoms = tuple(M + T for M in poly.monoms)

                        if order != poly.order:
                            terms = dict(zip(monoms, coeffs))
                    else:
                        terms = Poly._permute(poly, *symbols)
        else:
            terms = Poly._decompose(poly, *symbols)

        if terms:
            if N == 1 and type(terms.keys()[0]) is not tuple:
                keys = tuple(reversed(sorted(terms.keys())))

                coeffs = [ terms[i] for i in keys ]
                monoms = [ (i,) for i in keys ]
            else:
                f = monomial_cmp(order)

                monoms = terms.keys()
                monoms.sort(f, reverse=True)

                coeffs = [ terms[monom] for monom in monoms ]

        args = (tuple(coeffs), tuple(monoms),
                symbols, order, stamp, None)

        return Basic.__new__(cls, *args, **flags)

    def __getnewargs__(self):
        coeffs, monoms, symbols, order = self.args[:4]
        return ((coeffs,monoms),) + symbols

    @staticmethod
    def _permute(poly, *symbols):
        terms, N = {}, len(symbols)

        if symbols == poly.symbols[:N]:
            symbols = poly.symbols[N:]

            for coeff, monom in poly.iter_terms():
                coeff *= monomial_as_basic(monom[N:], *symbols)

                monom = monom[:N]

                if terms.has_key(monom):
                    coeff += terms[monom]

                    if not coeff:
                        del terms[monom]
                        continue

                terms[monom] = coeff
        else:
            new_symbols, indices = list(poly.symbols), []

            for s in new_symbols[:]:
                for i, t in enumerate(symbols):
                    if s is t:
                        new_symbols.remove(s)
                        indices.append(i)
                        break
                else:
                    indices.append(None)

            for coeff, monom in poly.iter_terms():
                exponents, j = [0] * N, 0

                for exp, i in zip(monom, indices):
                    if i is None:
                        coeff *= new_symbols[j]**exp
                        j += 1
                    else:
                        exponents[i] = exp

                monom = tuple(exponents)

                if terms.has_key(monom):
                    coeff += terms[monom]

                    if not coeff:
                        del terms[monom]
                        continue

                terms[monom] = coeff

        return terms

    @staticmethod
    def _decompose(poly, *symbols):
        """Converts basic expression to dictionary representation.

           This procedure will expand given expression and scan it,
           extracting coefficients and monomials according to a set
           of symbols, if possible. At this point there is no need
           for specifying ordering information. If conversion is
           not possible an exception is raised.

           This method should be left for internal use. To convert
           expression to a polynomial use Polynomial constructor
           explicitly or Basic.as_poly method.

           >>> from sympy import *
           >>> x, y = symbols('xy')

           >>> Poly._decompose(y*x**2 + 3, x)
           {(0,): 3, (2,): y}

           >>> Poly._decompose(y*x**2 + 3, x, y)
           {(0, 0): 3, (2, 1): 1}

        """
        result, indices, N = {}, {}, len(symbols)

        for i, sym in enumerate(symbols):
            indices[sym] = i

        poly = sympify(poly).expand()

        if poly.is_Add:
            terms = poly.args
        else:
            if poly.is_Number:
                return { (0,) * N : poly }
            else:
                terms = [ poly ]

        for term in terms:
            if not term.has_any_symbols(*symbols):
                coeff, monom = term, (0,) * N
            else:
                if term.is_Mul:
                    factors = term.args
                else:
                    factors = [ term ]

                coeff, monom = S.One, [0] * N

                for factor in factors:
                    if factor.has_any_symbols(*symbols):
                        if factor.is_Pow:
                            if factor.exp.is_Integer:
                                b, e = factor.base, factor.exp.p

                                if b.is_Symbol and e > 0:
                                    monom[indices[b]] += e
                                    continue
                        elif factor.is_Symbol:
                            monom[indices[factor]] += 1
                            continue

                        raise PolynomialError("Can't decompose %s" % factor)
                    else:
                        coeff *= factor

                monom = tuple(monom)

            if result.has_key(monom):
                coeff += result[monom]

                if not coeff:
                    del result[monom]
                    continue

            result[monom] = coeff

        if not result:
            return { (0,) * N : S.One }
        else:
            return result

    @staticmethod
    def cancel(f, *symbols):
        """Cancel common factors in a fractional expression.

           Given a quotient of polynomials, cancel common factors from
           the numerator and the denominator.  The result is formed in
           an expanded form, even if there was no cancellation.

           To improve the  speed of calculations a list of symbols can
           be specified. This can be particularly useful in univariate
           case with additional parameters, to avoid Groebner basis
           computations.

           The input fraction can be given as a single expression or
           as a tuple consisting of the numerator and denominator.

           >>> from sympy import *
           >>> x,y = symbols('xy')

           >>> Poly.cancel((x**2-y**2)/(x-y), x, y)
           x + y

           >>> Poly.cancel((x**2-y**2, x-y), x, y)
           x + y

        """
        if type(f) is tuple:
            numer, denom = f
        else:
            if f.is_Atom:
                return f

            if not symbols:
                symbols = f.atoms(Symbol)

                if not symbols:
                    symbols = None
                else:
                    symbols = sorted(symbols)

            numer, denom = f.as_numer_denom()

        numer = numer.expand()

        if numer is S.Zero:
            return numer

        denom = denom.expand()

        if denom.is_Number:
            return numer / denom

        if symbols is None:
            if denom.is_Add:
                a, b, c = map(Wild, 'abc')

                r = denom.match(a + b*c**S.Half)

                if r is not None:
                    if r[b] is S.Zero:
                        denom = match[a]
                    else:
                        a, b, c = r[a], r[b], r[c]

                        numer *= a-b*c**S.Half
                        numer = numer.expand()

                        denom = a**2 - c*b**2

            return numer / denom

        try:
            p = Poly(numer, *symbols)
            q = Poly(denom, *symbols)
        except PolynomialError:
            return numer / denom

        g = sympy.polys.algorithms.poly_gcd(p, q)

        p = sympy.polys.algorithms.poly_div(p, g)[0]
        q = sympy.polys.algorithms.poly_div(q, g)[0]

        pc = int(p.content)
        qc = int(q.content)

        cont = igcd(pc, qc)

        if cont != 1:
            p = p.div_term(cont)
            q = q.div_term(cont)

        return p.as_basic() / q.as_basic()

    def as_basic(self):
        """Converts polynomial to a valid sympy expression.

           Takes list representation of a polynomial and constructs
           expression using symbols used during creation of a given
           polynomial. If you wish to have different symbols in the
           resulting expressions, use subs() first.

           Note that the result is always returned in expanded form.

           >>> from sympy import *
           >>> x,y = symbols('xy')

           >>> p = Poly(x**2+3, x)
           >>> p.as_basic()
           3 + x**2

        """
        multinomial = dict(zip(self.monoms, self.coeffs))
        return multinomial_as_basic(multinomial, *self.symbols)

    def as_dict(self):
        """Convert list representation to dictionary representation.

           Due to hashing of immutable expressions in Python, we can
           not store a dictionary directly in a polynomial instance.
           Also it would be inconvenient in case of ordering of
           monomials (unnecessary sorting).

           This method creates dictionary efficiently, so it can be
           used in several algorithms which perform best when dicts
           are being used, eg. addition, multiplication etc.

           >>> from sympy import *
           >>> x,y = symbols('xy')

           >>> p = Poly(x**2+3, x)
           >>> p.as_dict()
           {(0,): 3, (2,): 1}

        """
        return dict(zip(self.monoms, self.coeffs))

    def as_uv_dict(self):
        """Return dictionary representation with integer keys.

           In case of univariate polynomials it is inefficient to
           to handle exponents as singleton tuples, as an example
           see multiplication algorithm.  This method will return
           dictionary representation with all tuples converted to
           explicit integers.

           >>> from sympy import *
           >>> x,y = symbols('xy')

           >>> p = Poly(x**2+3, x)
           >>> p.as_uv_dict()
           {0: 3, 2: 1}

        """
        if self.is_univariate:
            return dict(zip([ M[0] for M in self.monoms ], self.coeffs))
        else:
            raise MultivariatePolyError(self)

    @property
    def coeffs(self):
        return self._args[0]

    @property
    def monoms(self):
        return self._args[1]

    @property
    def symbols(self):
        return self._args[2]

    @property
    def order(self):
        return self._args[3]

    @property
    def stamp(self):
        return self._args[4]

    @property
    def domain(self):
        return self._args[5]

    @property
    def args(self):
        return self._args[0:4]

    @property
    def flags(self):
        return { 'order' : self.order }

    @property
    def length(self):
        return len(self.monoms)

    @property
    def degree(self):
        """Returns degree of the leading term. """
        return sum(self.monoms[0])

    @property
    def total_degree(self):
        """Returns true total degree of a polynomial. """
        return max([ sum(monom) for monom in self.monoms ])

    @property
    def norm(self):
        """Computes oo-norm of polynomial's coefficients. """
        return max([ abs(coeff) for coeff in self.coeffs ])

    def iter_basic_args(self):
        for coeff in self.coeffs:
            yield coeff

        for i, symbol in enumerate(self.symbols):
            if any(monom[i] != 0 for monom in self.monoms):
                yield symbol

    def iter_coeffs(self):
        for coeff in self.coeffs:
            yield coeff

    def iter_monoms(self):
        for monom in self.monoms:
            yield monom

    def iter_terms(self):
        for coeff, monom in zip(self.coeffs, self.monoms):
            yield coeff, monom

    def iter_all_coeffs(self):
        if self.is_univariate:
            terms = self.as_uv_dict()

            for i in xrange(self.degree, -1, -1):
                if terms.has_key(i):
                    yield terms[i]
                else:
                    yield S.Zero
        else:
            raise MultivariatePolyError(self)

    def iter_all_monoms(self):
        if self.is_univariate:
            for i in xrange(self.degree, -1, -1):
                yield (i,)
        else:
            raise MultivariatePolyError(self)

    def iter_all_terms(self):
        if self.is_univariate:
            terms = self.as_uv_dict()

            for i in xrange(self.degree, -1, -1):
                if terms.has_key(i):
                    yield terms[i], (i,)
                else:
                    yield S.Zero, (i,)
        else:
            raise MultivariatePolyError(self)

    @property
    def lead_coeff(self):
        return self.coeffs[0]

    @property
    def lead_monom(self):
        return self.monoms[0]

    @property
    def lead_term(self):
        return (self.coeffs[0], self.monoms[0])

    LC = lead_coeff
    LM = lead_monom
    LT = lead_term

    @property
    def tail_coeff(self):
        return self.coeffs[-1]

    @property
    def tail_monom(self):
        return self.monoms[-1]

    @property
    def tail_term(self):
        return (self.coeffs[-1], self.monoms[-1])

    TC = tail_coeff
    TM = tail_monom
    TT = tail_term

    def __abs__(self):
        return self.__class__(([ abs(coeff) for coeff in self.coeffs ],
            self.monoms), *self.symbols, **self.flags)

    def __neg__(self):
        return self.__class__(([ -coeff for coeff in self.coeffs ],
            self.monoms), *self.symbols, **self.flags)

    def __add__(self, other):
        try:
            self, poly = self.unify_with(other)
        except PolynomialError:
            return self.as_basic() + other.as_basic()

        if poly.is_zero:
            return self

        if self.is_zero:
            return poly

        if poly.is_monomial:
            return self.add_term(*poly.LT)

        if self.is_monomial:
            return poly.add_term(*self.LT)

        terms = self.as_dict()

        for coeff, monom in poly.iter_terms():
            if terms.has_key(monom):
                coeff += terms[monom]

                coeff = Poly.cancel(coeff)

                if not coeff:
                    del terms[monom]
                    continue

            terms[monom] = coeff

        return self.__class__(terms, *self.symbols, **self.flags)

    def __sub__(self, other):
        try:
            self, poly = self.unify_with(other)
        except PolynomialError:
            return self.as_basic() - other.as_basic()

        if poly.is_zero:
            return self

        if self.is_zero:
            return -poly

        if poly.is_monomial:
            return self.sub_term(*poly.LT)

        if self.is_monomial:
            return (-poly).add_term(*self.LT)

        terms = self.as_dict()

        for coeff, monom in poly.iter_terms():
            if terms.has_key(monom):
                coeff -= terms[monom]

                coeff = Poly.cancel(coeff)

                if not coeff:
                    del terms[monom]
                    continue

            terms[monom] = -coeff

        return self.__class__(terms, *self.symbols, **self.flags)

    def __mul__(self, other):
        """Efficiently multiply sparse and dense polynomials.

           Given polynomials P and Q, if both of them are dense, then
           in univariate case perform dense multiplication, otherwise
           apply Kronecker 'trick', mapping multivariate polynomials
           to univariate ones in last variable and then multiply.

           If any of the polynomials is sparse then in both univariate
           and multivariate cases use naive multiplication algorithm.

           For more information on implemented algorithms refer to:

           [1] R. T. Moenck, Practical Fast Polynomial Multiplication,
               Proceedings of the third ACM symposium on Symbolic and
               algebraic computation, ACM Press, Yorktown Heights,
               New York, USA, 1976, pp. 136-148
        """
        try:
            self, poly = self.unify_with(other)
        except PolynomialError:
            return self.as_basic() * other.as_basic()

        if self.is_constant:
            if self.is_zero:
                return self
            elif self.is_one:
                return poly
            else:
                return poly.mul_term(self.LC)

        if poly.is_constant:
            if poly.is_zero:
                return poly
            elif poly.is_one:
                return self
            else:
                return self.mul_term(poly.LC)

        if self.is_monomial:
            return poly.mul_term(*self.LT)

        if poly.is_monomial:
            return self.mul_term(*poly.LT)

        if self.is_dense and poly.is_dense:
            if self.is_multivariate:
                a = monomial_max(*self.monoms)
                b = monomial_max(*poly.monoms)

                degs, p, q = [1], {}, {}

                for x, y in reversed(zip(a, b)[1:]):
                    degs.insert(0, degs[0]*(x+y+1))

                N = sum([ x*y for x, y in zip(self.LM, degs) ])
                M = sum([ x*y for x, y in zip(poly.LM, degs) ])

                for coeff, monom in self.iter_terms():
                    p[sum([ x*y for x, y in zip(monom, degs) ])] = coeff

                for coeff, monom in poly.iter_terms():
                    q[sum([ x*y for x, y in zip(monom, degs) ])] = coeff
            else:
                p, N = self.as_uv_dict(), self.degree
                q, M = poly.as_uv_dict(), poly.degree

            coeffs, monoms = [], []

            for k in xrange(N+M, -1, -1):
                coeff = 0

                for i in xrange(k+1):
                    if p.has_key(i) and q.has_key(k-i):
                        coeff += Poly.cancel(p[i] * q[k-i])

                if coeff:
                    coeffs.append(coeff)
                    monoms.append((k,))

            if self.is_multivariate:
                terms, degs = {}, degs[:-1]

                for i, d in enumerate(monoms):
                    monom, d = [], d[0]

                    for j in degs:
                        k, d = divmod(d, j)
                        monom.append(k)

                    terms[tuple(monom)+(d,)] = coeffs[i]
            else:
                terms = coeffs, monoms
        else:
            terms = {}

            for coeff_p, M in self.iter_terms():
                for coeff_q, N in poly.iter_terms():
                    monom = monomial_mul(M, N)

                    coeff = coeff_p * coeff_q
                    coeff = Poly.cancel(coeff)

                    if terms.has_key(monom):
                        coeff += terms[monom]

                        if not coeff:
                            del terms[monom]
                            continue

                    terms[monom] = coeff

        return self.__class__(terms, *self.symbols, **self.flags)

    def __pow__(self, other):
        """Polynomial exponentiation using binary method.

           Given a polynomial P and integer exponent N, raise P to power
           N in floor(lg(N)) + 1(N) multiplications,  where lg(N) stands
           for binary logarithm and 1(N) is the number of ones in binary
           representation of N.

           For more information on the implemented algorithm refer to:

           [1] D. E. Knuth, The Art of Computer Programming: Seminumerical
               Algorithms, v.2, Addison-Wesley Professional, 1997, pp. 461

        """
        other = sympify(other)

        if isinstance(other, Poly):
            if other.is_constant:
                other = sympify(other.lead_coeff)
            else:
                return self.as_basic()**other.as_basic()

        if other.is_Integer:
            if other is S.One:
                return self

            P = self.__class__(S.One, *self.symbols, **self.flags)

            if other is S.Zero:
                return P

            N, Q = abs(int(other)), self

            while True:
                N, M = N//2, N

                if M & 1:
                    P *= Q

                    if N == 0:
                        break

                Q *= Q

            if other.is_positive:
                return P
            else:
                return Pow(P, S.NegativeOne)
        else:
            return self.as_basic()**other

    def __div__(self, other):
        try:
            return sympy.polys.algorithms.poly_div(self, other)[0]
        except PolynomialError:
            return self.as_basic() / other.as_basic()

    def __mod__(self, other):
        try:
            return sympy.polys.algorithms.poly_div(self, other)[1]
        except PolynomialError:
            return S.Zero

    def __divmod__(self, other):
        try:
            return sympy.polys.algorithms.poly_div(self, other)
        except PolynomialError:
            return (self.as_basic() / other.as_basic(), S.Zero)

    __truediv__ = __div__

    @property
    def is_zero(self):
        return self.coeffs in ((S.Zero,), (0,))

    @property
    def is_one(self):
        return self.coeffs in ((S.One,), (1,)) and \
            all(e == 0 for e in self.monoms[0])

    @property
    def is_number(self):
        return self.is_constant and self.coeffs[0].is_number

    @property
    def is_constant(self):
        return len(self.monoms) == 1 and \
            all(e == 0 for e in self.monoms[0])

    @property
    def is_monomial(self):
        return len(self.monoms) == 1

    @property
    def is_univariate(self):
        return len(self.symbols) == 1

    @property
    def is_multivariate(self):
        return len(self.symbols) != 1

    @property
    def is_homogeneous(self):
        return any(e != 0 for e in self.monoms[-1])

    @property
    def is_inhomogeneous(self):
        return all(e == 0 for e in self.monoms[-1])

    @property
    def is_sparse(self):
        return not self.is_dense

    @property
    def is_dense(self):
        """Returns True if 'self' is dense.

           Let 'self' be a polynomial in M variables of a total degree N
           and having L terms (with non-zero coefficients). Let K be the
           number of monomials in M variables of degree at most N.  Then
           'self' is considered dense if it's at least 90% filled.

           The total number of monomials is given by (M + N)! / (M! N!),
           where the factorials are estimated using improved Stirling's
           approximation:

                       n! = sqrt((2 n + 1/3) pi) n**n exp(-n)

           For more information on this subject refer to:

             http://mathworld.wolfram.com/StirlingsApproximation.html

        """
        def enum(M, N):
            U = float(M+N)
            V = (6*M+1)*(6*N+1)
            S = 2.4*math.sqrt(U/V)
            A = math.pow(U/M, M)
            B = math.pow(U/N, N)
            return S * A * B

        V, N = len(self.symbols), self.total_degree

        return (self.length / enum(V, N)) > 0.9

    @property
    def is_primitive(self):
        return self.content in (S.One, 1)

    @property
    def is_monic(self):
        return self.lead_coeff in (S.One, 1)

    @property
    def is_squarefree(self):
        """Returns True if 'self' has no factors of multiplicity >= 2.

           >>> from sympy import *
           >>> x,y = symbols('xy')

           >>> Poly(x-1, x).is_squarefree
           True

           >>> Poly((x-1)**2, x).is_squarefree
           False

        """
        return self.as_squarefree() is self

    @property
    def is_linear(self):
        return all(sum(monom) <= 1 for monom in self.iter_monoms())

    @property
    def has_integer_coeffs(self):
        return self.domain == 'Z'

    @property
    def has_rational_coeffs(self):
        return self.domain == 'Q'

    @property
    def has_radical_coeffs(self):
        return self.domain == 'R'

    @property
    def has_complex_coeffs(self):
        return self.domain == 'C'

    @property
    def has_symbolic_coeffs(self):
        return self.domain == 'S'

    def as_monic(self):
        """Returns a monic polynomial.

           >>> from sympy import *
           >>> x,y = symbols('xy')

           >>> Poly(x**2 + 4, x).as_monic()
           Poly(x**2 + 4, x)

           >>> Poly(2*x**2 + 4, x).as_monic()
           Poly(x**2 + 2, x)

           >>> Poly(y*x**2 + y**2 + y, x).as_monic()
           Poly(x**2 + 1 + y, x)

        """
        LC = self.lead_coeff

        if not self or LC is S.One:
            return self

        coeffs = [ coeff / LC for coeff in self.coeffs ]

        for i, coeff in enumerate(coeffs):
            coeffs[i] = Poly.cancel(coeff)

        return self.__class__((coeffs, self.monoms),
            *self.symbols, **self.flags)

    def as_integer(self):
        """Makes all coefficients integers if possible.

           Given a polynomial with rational coefficients,  returns a tuple
           consisting of a common denominator of those coefficients and an
           integer polynomial. Otherwise an exception is being raised.

           >>> from sympy import *
           >>> x,y = symbols("xy")

           >>> Poly(3*x**2 + x, x).as_integer()
           (1, Poly(3*x**2 + x, x))

           >>> Poly(3*x**2 + x/2, x).as_integer()
           (2, Poly(6*x**2 + x, x))

        """
        denom, coeffs = 1, []

        for coeff in self.iter_coeffs():
            if coeff.is_Rational:
                coeffs.append(coeff)

                if not coeff.is_Integer:
                    denom = ilcm(denom, coeff.q)
            elif coeff.is_Real and int(coeff) == coeff:
                coeffs.append(Integer(int(coeff)))
            else:
                raise CoefficientError("%s is not a rational number" % coeff)

        denom = sympify(denom)

        if denom is not S(1):
            coeffs = [ coeff * denom for coeff in self.iter_coeffs() ]

        return denom, self.__class__((coeffs, self.monoms),
            *self.symbols, **self.flags)

    @property
    def content(self):
        """Returns integer GCD of all the coefficients.

           >>> from sympy import *
           >>> x,y,z = symbols('xyz')

           >>> p = Poly(2*x + 5*x*y, x, y)
           >>> p.content
           1

           >>> p = Poly(2*x + 4*x*y, x, y)
           >>> p.content
           2

           >>> p = Poly(2*x + z*x*y, x, y)
           >>> p.content
           1

        """
        if self.is_zero:
            return S.One
        else:
            content = 0

            for coeff in self.coeffs:
                if coeff.is_Rational:
                    content = igcd(content, coeff.p)
                else:
                    return S.One

            return Integer(content)

    def as_primitive(self):
        """Returns content and primitive part of a polynomial.

           >>> from sympy import *
           >>> x,y = symbols('xy')

           >>> p = Poly(4*x**2 + 2*x, x)
           >>> p.as_primitive()
           (2, Poly(2*x**2 + x, x))

        """
        content = self.content

        if content is S.Zero or content is S.One:
            return content, self
        else:
            coeffs = [ coeff / content for coeff in self.coeffs ]

            return content, self.__class__((coeffs,
                self.monoms), *self.symbols, **self.flags)

    def as_squarefree(self):
        """Returns square-free part of a polynomial.

           >>> from sympy import *
           >>> x,y = symbols('xy')

           >>> Poly((x-1)**2, x).as_squarefree()
           Poly(x - 1, x)

        """
        f, A = self, sympy.polys.algorithms

        if f.is_univariate:
            F = f.as_primitive()[1]

            h = f.diff()

            g = A.poly_gcd(F, h)
            r = A.poly_div(f, g)

            return r[0]
        else:
            raise MultivariatePolyError(f)

    def as_reduced(self):
        """Remove GCD of monomials from 'self'.

           >>> from sympy import *
           >>> x,y = symbols('xy')

           >>> Poly(x**3 + x, x).as_reduced()
           ((1,), Poly(x**2 + 1, x))

           >>> Poly(x**3*y+x**2*y**2, x, y).as_reduced()
           ((2, 1), Poly(x + y, x, y))

        """
        if self.is_inhomogeneous:
            return (0,)*len(self.symbols), self
        else:
            if self.is_univariate:
                gcd = self.monoms[-1]

                terms = self.coeffs, [ (monom - gcd[0],)
                    for (monom,) in self.monoms ]
            else:
                gcd = monomial_min(*self.monoms)

                if all(not n for n in gcd):
                    return gcd, self

                terms = {}

                for coeff, monom in self.iter_terms():
                    terms[monomial_div(monom, gcd)] = coeff

            return gcd, Poly(terms, *self.symbols, **self.flags)

    def map_coeffs(self, f, *args, **kwargs):
        """Apply a function to all the coefficients.

           >>> from sympy import *
           >>> x,y,u,v = symbols('xyuv')

           >>> p = Poly(x**2 + 2*x*y, x, y)
           >>> q = p.map_coeffs(lambda c: 2*c)

           >>> q.as_basic()
           4*x*y + 2*x**2

           >>> p = Poly(u*x**2 + v*x*y, x, y)
           >>> q = p.map_coeffs(expand, complex=True)

           >>> q.as_basic()
           x**2*(I*im(u) + re(u)) + x*y*(I*im(v) + re(v))

        """
        terms = []

        for coeff, monom in self.iter_terms():
            coeff = f(coeff, *args, **kwargs)

            if coeff.has_any_symbols(*self.symbols):
                raise CoefficientError("%s coefficient is dependent" \
                    " of polynomial's symbols %s" % (coeff, self.symbols))
            elif coeff is not S.Zero:
                terms.append((coeff, monom))

        return self.__class__(terms, *self.symbols, **self.flags)

    def coeff(self, *monom):
        """Returns coefficient of a particular monomial.

           >>> from sympy import *
           >>> x,y = symbols('xy')

           >>> p = Poly(3*x**2*y + 4*x*y**2 + 1, x, y)

           >>> p.coeff(2, 1)
           3
           >>> p.coeff(1, 2)
           4
           >>> p.coeff(1, 1)
           0

           If len(monom) == 0 then returns coeff of the constant term:

           >>> p.coeff(0, 0)
           1
           >>> p.coeff()
           1

        """
        if not monom:
            if all(e == 0 for e in self.monoms[-1]):
                return self.coeffs[-1]
        else:
            for i in xrange(len(self.monoms)):
                if self.monoms[i] == monom:
                    return self.coeffs[i]

        return S.Zero

    def unify_with(self, other):
        """Unify 'self' with a polynomial or a set of polynomials.

           This method will return polynomials of the same type,  dominated
           by 'self', with a common set of symbols (which is a union of all
           symbols from all polynomials) and with common monomial ordering.

           You can pass a polynomial  or an expression to this method, or
           a list or a tuple of polynomials or expressions. If any of the
           inputs would be an expression then it will be converted to a
           polynomial.

        """
        symbols, stamp = self.symbols, self.stamp
        flags, cls = self.flags, self.__class__

        if isinstance(other, (tuple, list, set)):
            for poly in other:
                if isinstance(poly, Poly):
                    stamp |= poly.stamp
                #elif atoms:
                #    stamp |= poly.atoms(Symbol)

            if not (stamp <= self.stamp):
                symbols = tuple(sorted(stamp))
                self = cls(self, *symbols, **flags)

            other = other.__class__( cls(poly,
                *symbols, **flags) for poly in other )
        else:
            other = sympify(other)

            if isinstance(other, Poly):
                stamp |= other.stamp
            #elif atoms:
            #    stamp |= other.atoms(Symbol)

            if not (stamp <= self.stamp):
                symbols = tuple(sorted(stamp))
                self = cls(self, *symbols, **flags)

            other = cls(other, *symbols, **flags)

        return self, other

    def diff(self, *symbols):
        """Efficiently differentiate polynomials.

           Differentiate a polynomial with respect to a set of symbols.  If
           a symbol is polynomial's variable, then monomial differentiation
           is performed and coefficients updated with integer factors. In
           other case each of the coefficients is differentiated.

           Additionally, for each of the symbols you can specify a single
           positive integer which will indicate the number of times to
           perform differentiation using this symbol.

           >>> from sympy import *
           >>> x,y,z = symbols('xyz')

           >>> p = Poly(x**2*y + z**2, x, y)

           >>> p.diff(x)
           Poly(2*x*y, x, y)

           >>> p.diff(x, 2)
           Poly(2*y, x, y)

           >>> p.diff(x, 2, y)
           Poly(2, x, y)

           >>> p.diff(z)
           Poly(2*z, x, y)

        """
        if self.is_zero:
            return self

        new_symbols = []

        for s in symbols:
            s = sympify(s)

            if s.is_Symbol:
                new_symbols.append((s, 1))
            elif s.is_Integer:
                sym, _ = new_symbols.pop()
                new_symbols.append((sym, int(s)))
            else:
                raise TypeError

        if not new_symbols:
            if self.is_univariate:
                new_symbols = [(self.symbols[0], 1)]
            else:
                return self

        indices, symbols = {}, self.stamp

        for i, s in enumerate(self.symbols):
            indices[s] = i

        poly = self.as_dict()

        for s, k in new_symbols:
            new_poly = {}

            if s in symbols:
                i = indices[s]

                for M, coeff in poly.iteritems():
                    n = M[i] - k

                    if n >= 0:
                        monom = M[:i]+(n,)+M[i+1:]

                        for j in xrange(n, M[i]):
                            coeff *= j+1

                        if new_poly.has_key(monom):
                            coeff += new_poly[monom]

                            if not coeff:
                                del new_poly[monom]
                                continue

                        new_poly[monom] = coeff
            elif not isinstance(self, IntegerPoly):
                for monom, coeff in poly.iteritems():
                    if coeff.has_any_symbols(s):
                        coeff = coeff.diff(*([s]*k))

                        if coeff:
                            new_poly[monom] = coeff

            poly = new_poly

            if not poly:
                break

        return self.__class__(poly, *self.symbols, **self.flags)

    def integrate(self, *symbols):
        """Efficiently integrate polynomials.

           Integrate a polynomial with respect a set of symbols. If a
           symbol is polynomial's variable, then monomial integration
           is performed and coefficients updated with integer factors.
           In other case each of the coefficients is integrated.

           Additionally, for each of the symbols you can specify a
           single positive integer which will indicate the number
           of times to perform integration using this symbol.

           >>> from sympy import *
           >>> x,y,z = symbols('xyz')

           >>> p = Poly(x**2*y + z**2, x, y)

           >>> p.integrate(x)
           Poly(1/3*x**3*y + z**2*x, x, y)

           >>> p.integrate(x, 2)
           Poly(1/12*x**4*y + z**2/2*x**2, x, y)

           >>> p.integrate(x, 2, y)
           Poly(1/24*x**4*y**2 + z**2/2*x**2*y, x, y)

           >>> p.integrate(z)
           Poly(z*x**2*y + z**3/3, x, y)

        """
        if self.is_zero:
            return self

        new_symbols = []

        for s in symbols:
            s = sympify(s)

            if s.is_Symbol:
                new_symbols.append((s, 1))
            elif s.is_Integer:
                sym, _ = new_symbols.pop()
                new_symbols.append((sym, int(s)))
            else:
                raise TypeError

        indices, symbols = {}, self.stamp

        for i, s in enumerate(self.symbols):
            indices[s] = i

        poly = self.as_dict()

        for s, k in new_symbols:
            new_poly = {}

            if s in symbols:
                i = indices[s]

                for M, coeff in poly.iteritems():
                    n = M[i] + k

                    monom = M[:i]+(n,)+M[i+1:]

                    for j in xrange(M[i], n):
                        coeff /= j+1

                    if new_poly.has_key(monom):
                        coeff += new_poly[monom]

                        if not coeff:
                            del new_poly[monom]
                            continue

                    new_poly[monom] = coeff
            else:
                for M, coeff in poly.iteritems():
                    new_poly[M] = C.Integral(coeff, *([s]*k)).doit()

            if not new_poly:
                break
            else:
                poly = new_poly

        return self.__class__(poly, *self.symbols, **self.flags)

    def invert(self, modulus):
        """Compute multiplicative inverse in K[x]. """
        self, modulus = self.unify_with(modulus)

        if self.is_multivariate:
            raise PolynomialError("only univariate polynomials can be inverted")

        s, g = sympy.polys.algorithms.poly_half_gcdex(self, modulus)

        if g.is_one:
            return sympy.polys.algorithms.poly_div(s, modulus)[1]
        else:
            raise ZeroDivisionError("polynomial division")


    def add_term(self, coeff, monom):
        """Efficiently add a single term to 'self'.

           The list of monomials is sorted at initialization time, this
           motivates usage of binary search algorithm to find an index
           of an existing monomial or a suitable place for a new one.
           This gives O(lg(n)) complexity, where 'n' is the initial
           number of terms, superior to naive approach.

           For more information on the implemented algorithm refer to:

           [1] D. E. Knuth, The Art of Computer Programming: Sorting
               and Searching, v.1, Addison-Wesley Professional, 1998

        """
        if self.is_zero:
            coeffs = (coeff,)
            monoms = (monom,)
        else:
            coeffs = list(self.coeffs)
            monoms = list(self.monoms)

            compare = monomial_cmp(self.order)

            if compare(monom, monoms[0]) > 0:
                coeffs.insert(0, coeff)
                monoms.insert(0, monom)
            elif compare(monom, monoms[-1]) < 0:
                coeffs.append(coeff)
                monoms.append(monom)
            else:
                lo, hi = 0, len(monoms)-1

                while lo <= hi:
                    i = (lo + hi) // 2

                    k = compare(monom, monoms[i])

                    if not k:
                        coeff += coeffs[i]

                        if coeff:
                            coeffs[i] = coeff
                        else:
                            del coeffs[i]
                            del monoms[i]

                        break
                    else:
                        if k > 0:
                            hi = i - 1
                        else:
                            lo = i + 1
                else:
                    coeffs.insert(i, coeff)
                    monoms.insert(i, monom)

        return self.__class__((coeffs, monoms),
            *self.symbols, **self.flags)

    def sub_term(self, coeff, monom):
        """Efficiently subtract a single term from 'self'.

           The list of monomials is sorted at initialization time, this
           motivates usage of binary search algorithm to find an index
           of an existing monomial or a suitable place for a new one.
           This gives O(lg(n)) complexity, where 'n' is the initial
           number of terms, superior to naive approach.

           For more information on the implemented algorithm refer to:

           [1] D. E. Knuth, The Art of Computer Programming: Sorting
               and Searching, v.2, Addison-Wesley Professional, 1998

        """
        if self.is_zero:
            coeffs = (-coeff,)
            monoms = (monom,)
        else:
            coeffs = list(self.coeffs)
            monoms = list(self.monoms)

            compare = monomial_cmp(self.order)

            if compare(monom, monoms[0]) > 0:
                coeffs.insert(0, -coeff)
                monoms.insert(0, monom)
            elif compare(monom, monoms[-1]) < 0:
                coeffs.append(-coeff)
                monoms.append(monom)
            else:
                lo, hi = 0, len(monoms)-1

                while lo <= hi:
                    i = (lo + hi) // 2

                    k = compare(monom, monoms[i])

                    if not k:
                        coeff -= coeffs[i]

                        if coeff:
                            coeffs[i] = -coeff
                        else:
                            del coeffs[i]
                            del monoms[i]

                        break
                    else:
                        if k > 0:
                            hi = i - 1
                        else:
                            lo = i + 1
                else:
                    coeffs.insert(i, -coeff)
                    monoms.insert(i, monom)

        return self.__class__((coeffs, monoms),
            *self.symbols, **self.flags)

    def mul_term(self, coeff, monom=None):
        """Efficiently multiply 'self' with a single term. """
        coeff = sympify(coeff)

        if coeff is S.Zero:
            terms = ()
        else:
            if coeff is S.One:
                if monom is None:
                    return self
                else:
                    coeffs = self.coeffs
            else:
                coeffs = [ p_coeff * coeff for p_coeff in self.coeffs ]

                for i, coeff in enumerate(coeffs):
                    coeffs[i] = Poly.cancel(coeff)

            if monom is None:
                monoms = self.monoms
            else:
                monoms = [ monomial_mul(p_monom, monom)
                    for p_monom in self.monoms ]

            terms = coeffs, monoms

        return self.__class__(terms, *self.symbols, **self.flags)

    def div_term(self, coeff, monom=None):
        """Efficiently divide 'self' by a single term. """
        coeff = sympify(coeff)

        if coeff is S.Zero:
            raise ZeroDivisionError

        if coeff is S.One:
            if monom is None:
                return self
            else:
                coeffs = self.coeffs
        else:
            coeffs = [ p_coeff / coeff for p_coeff in self.coeffs ]

            for i, coeff in enumerate(coeffs):
                coeffs[i] = Poly.cancel(coeff)

        if monom is None:
            monoms = self.monoms
        else:
            monoms = [ monomial_div(p_monom, monom)
                for p_monom in self.monoms ]

            if any(monom is None for monom in monoms):
                raise PolynomialError("%s monomial must divide" \
                    " exactly the given polynomial" % (monom,))

        return self.__class__((coeffs, monoms),
            *self.symbols, **self.flags)

    def kill_lead_term(self):
        """Removes leading term from 'self'. """
        terms = self.coeffs[1:], self.monoms[1:]
        return self.__class__(terms, *self.symbols, **self.flags)

    def kill_last_term(self):
        """Removes last term from 'self'. """
        terms = self.coeffs[:-1], self.monoms[:-1]
        return self.__class__(terms, *self.symbols, **self.flags)

    def __call__(self, *point):
        """Efficiently evaluate polynomial at a given point.

           Evaluation is always done using Horner scheme.  In multivariate
           case a greedy algorithm is used to obtain a sequence of partial
           evaluations which minimizes the total number of multiplications
           required to perform this evaluation. This strategy is efficient
           for most of multivariate polynomials.

           Note that evaluation is done for all variables, which means the
           dimension of the given point must match the number of symbols.

           If you wish to efficiently evaluate polynomial for a subset of
           symbols use 'evaluate' method instead. Alternatively one can
           use Basic.subs() for this purpose.

           >>> from sympy import *
           >>> x,y = symbols('xy')

           >>> p = Poly(x**2 + 2*x + 1, x)

           >>> p(2)
           9
           >>> p(y)
           1 + y*(2 + y)

           For more information on the implemented algorithm refer to:

           [1] M. Ceberio, V. Kreinovich, Greedy Algorithms for Optimizing
               Multivariate Horner Schemes, ACM SIGSAM Bulletin, Volume 38,
               Issue 1, 2004, pp. 8-15

        """
        N = len(self.symbols)

        if len(point) != N:
            raise ValueError("Dimension of %s does not match" \
                " the number of symbols (%d)" % (point, N))

        if self.is_univariate:
            terms = self.as_uv_dict()

            point, result = point[0], 0

            for k in xrange(self.degree, -1, -1):
                result *= point

                if terms.has_key(k):
                    result += terms[k]

            return result
        else:
            def evaluate(terms):
                count = [0] * N

                for monom in terms.iterkeys():
                    for i, M in enumerate(monom):
                        if M != 0:
                            count[i] += 1

                K = max(count)

                if K <= 1:
                    result = 0

                    for monom, coeff in terms.iteritems():
                        for base, exp in zip(point, monom):
                            if exp != 0:
                                if exp == 1:
                                    coeff *= base
                                else:
                                    coeff *= base**exp

                        result += coeff

                    return result
                else:
                    k, indeps, depend = count.index(K), {}, {}

                    n = min([ M[k] for M in terms.iterkeys() if M[k] ])

                    for M, coeff in terms.iteritems():
                        if M[k] != 0:
                            depend[M[:k]+(M[k]-n,)+M[k+1:]] = coeff
                        else:
                            indeps[M] = coeff

                    result = point[k]**n * evaluate(depend)

                    if indeps:
                        return result + evaluate(indeps)
                    else:
                        return result

            return evaluate(self.as_dict())

    def evaluate(self, pattern):
        """Evaluates polynomial for a given set of symbols. """
        symbols = list(self.symbols)

        if type(pattern) is dict:
            pattern = pattern.items()
        elif type(pattern) is tuple:
            pattern = [pattern]

        poly = self.as_dict()

        for s, value in pattern:
            if s not in self.stamp:
                raise PolynomialError("%s symbol does" \
                    " not belong to %s" % (s, self.symbols))
            else:
                i = symbols.index(s)

            # 'value' might be int | long
            if isinstance(value, Symbol):
                if value in self.stamp:
                    raise PolynomialError("%s symbol must" \
                        " not belong to %s" % (s, self.symbols))

            terms = {}

            for M, coeff in poly.iteritems():
                monom = M[:i] + M[i+1:]
                coeff *= value ** M[i]

                coeff = Poly.cancel(coeff)

                if terms.has_key(monom):
                    coeff += terms[monom]

                    if not coeff:
                        del terms[monom]
                        continue
                elif not coeff:
                    continue

                terms[monom] = coeff

            del symbols[i]
            poly = terms

        if not symbols:
            if not poly:
                return S.Zero
            else:
                return poly.popitem()[1]
        else:
            return self.__class__(poly, *symbols, **self.flags)

    def _eval_subs(self, old, new):
        symbols = list(self.symbols)

        if old in self.stamp:
            terms, i = {}, symbols.index(old)

            if new.is_Symbol:
                if new in self.stamp:
                    j = symbols.index(new)

                    for coeff, M in self.iter_terms():
                        n, m = M[i], M[j]

                        if i < j:
                            monom = M[:i]+M[i+1:j]+(m+n,)+M[j+1:]
                        else:
                            monom = M[:j]+(m+n,)+M[j+1:i]+M[i+1:]

                        if terms.has_key(monom):
                            coeff += terms[monom]

                            if not coeff:
                                del terms[monom]
                                continue

                        terms[monom] = coeff

                    del symbols[i]

                    return self.__class__(terms, *symbols, **self.flags)
                else:
                    for coeff in self.coeffs:
                        if coeff.has_any_symbols(new):
                            break
                    else:
                        symbols[i], terms = new, (self.coeffs, self.monoms)
                        return self.__class__(terms, *symbols, **self.flags)
            elif new.is_number:
                if len(symbols) == 1:
                    return self(new)
                else:
                    return self.evaluate((old, new))
        elif not new.has_any_symbols(*symbols):
            coeffs = [ sympify(coeff)._eval_subs(old, new) for coeff in self.coeffs ]
            return self.__class__(zip(coeffs, self.monoms), *symbols, **self.flags)

        result = self.as_basic()._eval_subs(old, new)

        try:
            return self.__class__(result, *symbols, **self.flags)
        except PolynomialError:
            return result

    def __eq__(self, other):
        """Compare polynomials up to order of symbols and monomials. """
        try:
            poly = self.__class__(other, *self.symbols, **self.flags)
        except PolynomialError:
            return False

        if self.length != poly.length:
            return False

        if hash(self.monoms) != hash(poly.monoms):
            return False

        if hash(self.coeffs) != hash(poly.coeffs):
            for a, b in zip(self.coeffs, poly.coeffs):
                if a != b:
                    return False

        return True

    def __ne__(self, other):
        """Compare polynomials up to order of symbols and monomials. """
        try:
            poly = self.__class__(other, *self.symbols, **self.flags)
        except PolynomialError:
            return True

        if self.length != poly.length:
            return True

        if hash(self.monoms) != hash(poly.monoms):
            return True

        if hash(self.coeffs) != hash(poly.coeffs):
            for a, b in zip(self.coeffs, poly.coeffs):
                if a != b:
                    return True

        return False

    def __nonzero__(self):
        return self.coeffs not in ((S.Zero,), (0,))

    def _eval_is_polynomial(self, symbols):
        try:
            self.__class__(self, *symbols, **self.flags)
        except PolynomialError:
            return False

        return True

class IntegerPoly(Poly):
    pass

def multinomial_as_basic(multinomial, *symbols):
    """
    Converts the multinomial to Add/Mul/Pow instances.

    multinomial is a dict of {powers: coefficient} pairs, powers is a tuple of
    python integers, coefficient is a python integer.
    """
    l = []
    for powers, k in multinomial.iteritems():
        term = [k]
        for i,e in enumerate(powers):
            term.append(Pow(symbols[i], powers[i]))
        l.append(Mul(*term))
    result = Add(*l)
    return result




#!/usr/bin/env python
"""Plotting example

Demonstrates simple plotting.
"""

from sympy import Symbol, cos, sin, Plot, log, tan
from sympy.abc import x, y

def main():
    fun1 = cos(x)*sin(y)
    fun2 = sin(x)*sin(y)
    fun3 = cos(y) + log(tan(y/2)) + 0.2*x

    Plot(fun1, fun2, fun3, [x, -0.00, 12.4, 40], [y, 0.1, 2, 40])

if __name__ == "__main__":
    main()



"""
Adaptive numerical evaluation of SymPy expressions, using mpmath
for mathematical functions.
"""

from sympy.mpmath.libmpf import (from_int, from_rational, fzero, normalize,
        bitcount, round_nearest, to_str, fone, fnone, fhalf, to_float,
        from_float, fnone, to_int, mpf_lt, mpf_sqrt, mpf_cmp, mpf_abs,
        mpf_pow_int, mpf_shift, mpf_add, mpf_mul, mpf_neg)

import sympy.mpmath.libmpc as libmpc
from sympy.mpmath.settings import dps_to_prec
from sympy.mpmath import mpf, mpc, quadts, quadosc, mp, make_mpf
from sympy.mpmath.gammazeta import mpf_gamma
from sympy.mpmath.libelefun import mpf_pi, mpf_log, mpf_pow, mpf_sin, mpf_cos, \
        mpf_atan, mpf_atan2, mpf_e, mpf_exp
from sympy.mpmath.libmpf import MP_BASE, from_man_exp
from sympy.mpmath.calculus import shanks, richardson, nsum
from sympy.mpmath import inf as mpmath_inf

from sympy.mpmath.gammazeta import mpf_bernoulli

import math

from basic import Basic, C, S
from function import Function
from sympify import sympify

LG10 = math.log(10,2)

# Used in a few places as placeholder values to denote exponents and
# precision levels, e.g. of exact numbers. Must be careful to avoid
# passing these to mpmath functions or returning them in final results.
INF = 1e1000
MINUS_INF = -1e1000

# ~= 100 digits. Real men set this to INF.
DEFAULT_MAXPREC = 333

class PrecisionExhausted(ArithmeticError):
    pass

#----------------------------------------------------------------------------#
#                                                                            #
#              Helper functions for arithmetic and complex parts             #
#                                                                            #
#----------------------------------------------------------------------------#

"""
An mpf value tuple is a tuple of integers (sign, man, exp, bc)
representing a floating-point numbers.

A temporary result is a tuple (re, im, re_acc, im_acc) where
re and im are nonzero mpf value tuples representing approximate
numbers, or None to denote exact zeros.

re_acc, im_acc are integers denoting log2(e) where e is the estimated
relative accuracy of the respective complex part, but may be anything
if the corresponding complex part is None.

"""

def fastlog(x):
    """Fast approximation of log2(x) for an mpf value tuple x."""
    if not x or x == fzero:
        return MINUS_INF
    # log2(x) ~= exponent + width of mantissa
    # Note: this actually gives ceil(log2(x)), which is a useful
    # feature for interval arithmetic.
    return x[2] + x[3]

def complex_accuracy(result):
    """
    Returns relative accuracy of a complex number with given accuracies
    for the real and imaginary parts. The relative accuracy is defined
    in the complex norm sense as ||z|+|error|| / |z| where error
    is equal to (real absolute error) + (imag absolute error)*i.

    The full expression for the (logarithmic) error can be approximated
    easily by using the max norm to approximate the complex norm.

    In the worst case (re and im equal), this is wrong by a factor
    sqrt(2), or by log2(sqrt(2)) = 0.5 bit.
    """
    re, im, re_acc, im_acc = result
    if not im:
        if not re:
            return INF
        return re_acc
    if not re:
        return im_acc
    re_size = fastlog(re)
    im_size = fastlog(im)
    absolute_error = max(re_size-re_acc, im_size-im_acc)
    relative_error = absolute_error - max(re_size, im_size)
    return -relative_error

def get_abs(expr, prec, options):
    re, im, re_acc, im_acc = evalf(expr, prec+2, options)
    if not re:
        re, re_acc, im, im_acc = im, im_acc, re, re_acc
    if im:
        return libmpc.mpc_abs((re, im), prec), None, re_acc, None
    else:
        return mpf_abs(re), None, re_acc, None

def get_complex_part(expr, no, prec, options):
    """no = 0 for real part, no = 1 for imaginary part"""
    workprec = prec
    i = 0
    while 1:
        res = evalf(expr, workprec, options)
        value, accuracy = res[no::2]
        if (not value) or accuracy >= prec:
            return value, None, accuracy, None
        workprec += max(30, 2**i)
        i += 1

def evalf_abs(expr, prec, options):
    return get_abs(expr.args[0], prec, options)

def evalf_re(expr, prec, options):
    return get_complex_part(expr.args[0], 0, prec, options)

def evalf_im(expr, prec, options):
    return get_complex_part(expr.args[0], 1, prec, options)

def finalize_complex(re, im, prec):
    assert re and im
    if re == fzero and im == fzero:
        raise ValueError("got complex zero with unknown accuracy")
    size_re = fastlog(re)
    size_im = fastlog(im)
    # Convert fzeros to scaled zeros
    if re == fzero:
        re = mpf_shift(fone, size_im-prec)
        size_re = fastlog(re)
    elif im == fzero:
        im = mpf_shift(fone, size_re-prec)
        size_im = fastlog(im)
    if size_re > size_im:
        re_acc = prec
        im_acc = prec + min(-(size_re - size_im), 0)
    else:
        im_acc = prec
        re_acc = prec + min(-(size_im - size_re), 0)
    return re, im, re_acc, im_acc

def chop_parts(value, prec):
    """
    Chop off tiny real or complex parts.
    """
    re, im, re_acc, im_acc = value
    # Method 1: chop based on absolute value
    if re and (fastlog(re) < -prec+4):
        re, re_acc = None, None
    if im and (fastlog(im) < -prec+4):
        im, im_acc = None, None
    # Method 2: chop if inaccurate and relatively small
    if re and im:
        delta = fastlog(re) - fastlog(im)
        if re_acc < 2 and (delta - re_acc <= -prec+4):
            re, re_acc = None, None
        if im_acc < 2 and (delta - im_acc >= prec-4):
            im, im_acc = None, None
    return re, im, re_acc, im_acc

def check_target(expr, result, prec):
    a = complex_accuracy(result)
    if a < prec:
        raise PrecisionExhausted("Failed to distinguish the expression: \n\n%s\n\n"
            "from zero. Try simplifying the input, using chop=True, or providing "
            "a higher maxprec for evalf" % (expr))

def get_integer_part(expr, no, options, return_ints=False):
    """
    With no = 1, computes ceiling(expr)
    With no = -1, computes floor(expr)

    Note: this function either gives the exact result or signals failure.
    """

    # The expression is likely less than 2^30 or so
    assumed_size = 30
    ire, iim, ire_acc, iim_acc = evalf(expr, assumed_size, options)

    # We now know the size, so we can calculate how much extra precision
    # (if any) is needed to get within the nearest integer
    if ire and iim:
        gap = max(fastlog(ire)-ire_acc, fastlog(iim)-iim_acc)
    elif ire:
        gap = fastlog(ire)-ire_acc
    elif iim:
        gap = fastlog(iim)-iim_acc
    else:
        # ... or maybe the expression was exactly zero
        return None, None, None, None

    margin = 10

    if gap >= -margin:
        ire, iim, ire_acc, iim_acc = evalf(expr, margin+assumed_size+gap, options)

    # We can now easily find the nearest integer, but to find floor/ceil, we
    # must also calculate whether the difference to the nearest integer is
    # positive or negative (which may fail if very close)
    def calc_part(expr, nexpr):
        nint = int(to_int(nexpr, round_nearest))
        expr = C.Add(expr, -nint, evaluate=False)
        x, _, x_acc, _ = evalf(expr, 10, options)
        check_target(expr, (x, None, x_acc, None), 3)
        nint += int(no*(mpf_cmp(x or fzero, fzero) == no))
        nint = from_int(nint)
        return nint, fastlog(nint) + 10

    re, im, re_acc, im_acc = None, None, None, None

    if ire:
        re, re_acc = calc_part(C.re(expr, evaluate=False), ire)
    if iim:
        im, im_acc = calc_part(C.im(expr, evaluate=False), iim)

    if return_ints:
        return int(to_int(re or fzero)), int(to_int(im or fzero))
    return re, im, re_acc, im_acc

def evalf_ceiling(expr, prec, options):
    return get_integer_part(expr.args[0], 1, options)

def evalf_floor(expr, prec, options):
    return get_integer_part(expr.args[0], -1, options)

#----------------------------------------------------------------------------#
#                                                                            #
#                            Arithmetic operations                           #
#                                                                            #
#----------------------------------------------------------------------------#

def add_terms(terms, prec, target_prec):
    """
    Helper for evalf_add. Adds a list of (mpfval, accuracy) terms.
    """
    if len(terms) == 1:
        if not terms[0]:
            # XXX: this is supposed to represent a scaled zero
            return mpf_shift(fone, target_prec), -1
        return terms[0]
    sum_man, sum_exp, absolute_error = 0, 0, MINUS_INF
    for x, accuracy in terms:
        if not x:
            continue
        sign, man, exp, bc = x
        if sign:
            man = -man
        absolute_error = max(absolute_error, bc+exp-accuracy)
        delta = exp - sum_exp
        if exp >= sum_exp:
            if delta > 4*prec:
                sum_man = man
                sum_exp = exp
            else:
                sum_man += man << delta
        else:
            if (-delta) > 4*prec:
                pass
            else:
                sum_man = (sum_man << (-delta)) + man
                sum_exp = exp
    if absolute_error == MINUS_INF:
        return None, None
    if not sum_man:
        # XXX: this is supposed to represent a scaled zero
        return mpf_shift(fone, absolute_error), -1
    if sum_man < 0:
        sum_sign = 1
        sum_man = -sum_man
    else:
        sum_sign = 0
    sum_bc = bitcount(sum_man)
    sum_accuracy = sum_exp + sum_bc - absolute_error
    r = normalize(sum_sign, sum_man, sum_exp, sum_bc, target_prec,
        round_nearest), sum_accuracy
    #print "returning", to_str(r[0],50), r[1]
    return r

def evalf_add(v, prec, options):
    args = v.args
    target_prec = prec
    i = 0

    oldmaxprec = options.get('maxprec', DEFAULT_MAXPREC)
    options['maxprec'] = min(oldmaxprec, 2*prec)

    try:
        while 1:
            terms = [evalf(arg, prec+10, options) for arg in args]
            re, re_accuracy = add_terms([(a[0],a[2]) for a in terms if a[0]], prec, target_prec)
            im, im_accuracy = add_terms([(a[1],a[3]) for a in terms if a[1]], prec, target_prec)
            accuracy = complex_accuracy((re, im, re_accuracy, im_accuracy))
            if accuracy >= target_prec:
                if options.get('verbose'):
                    print "ADD: wanted", target_prec, "accurate bits, got", re_accuracy, im_accuracy
                return re, im, re_accuracy, im_accuracy
            else:
                diff = target_prec - accuracy
                if (prec-target_prec) > options.get('maxprec', DEFAULT_MAXPREC):
                    return re, im, re_accuracy, im_accuracy

                prec = prec + max(10+2**i, diff)
                options['maxprec'] = min(oldmaxprec, 2*prec)
                if options.get('verbose'):
                    print "ADD: restarting with prec", prec
            i += 1
    finally:
        options['maxprec'] = oldmaxprec

# Helper for complex multiplication
# XXX: should be able to multiply directly, and use complex_accuracy
# to obtain the final accuracy
def cmul((a, aacc), (b, bacc), (c, cacc), (d, dacc), prec, target_prec):
    A, Aacc = mpf_mul(a,c,prec), min(aacc, cacc)
    B, Bacc = mpf_mul(mpf_neg(b),d,prec), min(bacc, dacc)
    C, Cacc = mpf_mul(a,d,prec), min(aacc, dacc)
    D, Dacc = mpf_mul(b,c,prec), min(bacc, cacc)
    re, re_accuracy = add_terms([(A, Aacc), (B, Bacc)], prec, target_prec)
    im, im_accuracy = add_terms([(C, Cacc), (D, Cacc)], prec, target_prec)
    return re, im, re_accuracy, im_accuracy

def evalf_mul(v, prec, options):
    args = v.args
    # With guard digits, multiplication in the real case does not destroy
    # accuracy. This is also true in the complex case when considering the
    # total accuracy; however accuracy for the real or imaginary parts
    # separately may be lower.
    acc = prec
    target_prec = prec
    # XXX: big overestimate
    prec = prec + len(args) + 5
    direction = 0
    # Empty product is 1
    man, exp, bc = MP_BASE(1), 0, 1
    direction = 0
    complex_factors = []
    # First, we multiply all pure real or pure imaginary numbers.
    # direction tells us that the result should be multiplied by
    # i**direction
    for arg in args:
        re, im, a, aim = evalf(arg, prec, options)
        if re and im:
            complex_factors.append((re, im, a, aim))
            continue
        elif re:
            s, m, e, b = re
        elif im:
            a = aim
            direction += 1
            s, m, e, b = im
        else:
            return None, None, None, None
        direction += 2*s
        man *= m
        exp += e
        bc += b
        if bc > 3*prec:
            man >>= prec
            exp += prec
        acc = min(acc, a)
    sign = (direction & 2) >> 1
    v = normalize(sign, man, exp, bitcount(man), prec, round_nearest)
    if complex_factors:
        # Multiply first complex number by the existing real scalar
        re, im, re_acc, im_acc = complex_factors[0]
        re = mpf_mul(re, v, prec)
        im = mpf_mul(im, v, prec)
        re_acc = min(re_acc, acc)
        im_acc = min(im_acc, acc)
        # Multiply consecutive complex factors
        complex_factors = complex_factors[1:]
        for wre, wim, wre_acc, wim_acc in complex_factors:
            re, im, re_acc, im_acc = cmul((re, re_acc), (im,im_acc),
                (wre,wre_acc), (wim,wim_acc), prec, target_prec)
        if options.get('verbose'):
            print "MUL: obtained accuracy", re_acc, im_acc, "expected", target_prec
        # multiply by i
        if direction & 1:
            return mpf_neg(im), re, re_acc, im_acc
        else:
            return re, im, re_acc, im_acc
    else:
        # multiply by i
        if direction & 1:
            return None, v, None, acc
        else:
            return v, None, acc, None

def evalf_pow(v, prec, options):

    target_prec = prec
    base, exp = v.args

    # We handle x**n separately. This has two purposes: 1) it is much
    # faster, because we avoid calling evalf on the exponent, and 2) it
    # allows better handling of real/imaginary parts that are exactly zero
    if exp.is_Integer:
        p = exp.p
        # Exact
        if not p:
            return fone, None, prec, None
        # Exponentiation by p magnifies relative error by |p|, so the
        # base must be evaluated with increased precision if p is large
        prec += int(math.log(abs(p),2))
        re, im, re_acc, im_acc = evalf(base, prec+5, options)
        # Real to integer power
        if re and not im:
            return mpf_pow_int(re, p, target_prec), None, target_prec, None
        # (x*I)**n = I**n * x**n
        if im and not re:
            z = fpowi(im, p, target_prec)
            case = p % 4
            if case == 0: return z, None, target_prec, None
            if case == 1: return None, z, None, target_prec
            if case == 2: return mpf_neg(z), None, target_prec, None
            if case == 3: return None, mpf_neg(z), None, target_prec
        # Zero raised to an integer power
        if not re:
            return None, None, None, None
        # General complex number to arbitrary integer power
        re, im = libmpc.mpc_pow_int((re, im), p, prec)
        # Assumes full accuracy in input
        return finalize_complex(re, im, target_prec)

    # Pure square root
    if exp is S.Half:
        xre, xim, xre_acc, yim_acc = evalf(base, prec+5, options)
        # General complex square root
        if xim:
            re, im = libmpc.mpc_sqrt((xre or fzero, xim), prec)
            return finalize_complex(re, im, prec)
        if not xre:
            return None, None, None, None
        # Square root of a negative real number
        if mpf_lt(xre, fzero):
            return None, mpf_sqrt(mpf_neg(xre), prec), None, prec
        # Positive square root
        return mpf_sqrt(xre, prec), None, prec, None

    # We first evaluate the exponent to find its magnitude
    # This determines the working precision that must be used
    prec += 10
    yre, yim, yre_acc, yim_acc = evalf(exp, prec, options)
    # Special cases: x**0
    if not (yre or yim):
        return fone, None, prec, None

    ysize = fastlog(yre)
    # Restart if too big
    # XXX: prec + ysize might exceed maxprec
    if ysize > 5:
        prec += ysize
        yre, yim, yre_acc, yim_acc = evalf(exp, prec, options)

    # Pure exponential function; no need to evalf the base
    if base is S.Exp1:
        if yim:
            re, im = libmpc.mpc_exp((yre or fzero, yim), prec)
            return finalize_complex(re, im, target_prec)
        return mpf_exp(yre, target_prec), None, target_prec, None

    xre, xim, xre_acc, yim_acc = evalf(base, prec+5, options)
    # 0**y
    if not (xre or xim):
        return None, None, None, None

    # (real ** complex) or (complex ** complex)
    if yim:
        re, im = libmpc.mpc_pow((xre or fzero, xim or fzero), (yre or fzero, yim),
            target_prec)
        return finalize_complex(re, im, target_prec)
    # complex ** real
    if xim:
        re, im = libmpc.mpc_pow_mpf((xre or fzero, xim), yre, target_prec)
        return finalize_complex(re, im, target_prec)
    # negative ** real
    elif mpf_lt(xre, fzero):
        re, im = libmpc.mpc_pow_mpf((xre, fzero), yre, target_prec)
        return finalize_complex(re, im, target_prec)
    # positive ** real
    else:
        return mpf_pow(xre, yre, target_prec), None, target_prec, None




#----------------------------------------------------------------------------#
#                                                                            #
#                            Special functions                               #
#                                                                            #
#----------------------------------------------------------------------------#

def evalf_trig(v, prec, options):
    """
    This function handles sin and cos of real arguments.

    TODO: should also handle tan and complex arguments.
    """
    if v.func is C.cos:
        func = mpf_cos
    elif v.func is C.sin:
        func = mpf_sin
    else:
        raise NotImplementedError
    arg = v.args[0]
    # 20 extra bits is possibly overkill. It does make the need
    # to restart very unlikely
    xprec = prec + 20
    re, im, re_accuracy, im_accuracy = evalf(arg, xprec, options)
    if im:
        raise NotImplementedError
    if not re:
        if v.func is C.cos:
            return fone, None, prec, None
        elif v.func is C.sin:
            return None, None, None, None
        else:
            raise NotImplementedError
    # For trigonometric functions, we are interested in the
    # fixed-point (absolute) accuracy of the argument.
    xsize = fastlog(re)
    # Magnitude <= 1.0. OK to compute directly, because there is no
    # danger of hitting the first root of cos (with sin, magnitude
    # <= 2.0 would actually be ok)
    if xsize < 1:
        return func(re, prec, round_nearest), None, prec, None
    # Very large
    if xsize >= 10:
        xprec = prec + xsize
        re, im, re_accuracy, im_accuracy = evalf(arg, xprec, options)
    # Need to repeat in case the argument is very close to a
    # multiple of pi (or pi/2), hitting close to a root
    while 1:
        y = func(re, prec, round_nearest)
        ysize = fastlog(y)
        gap = -ysize
        accuracy = (xprec - xsize) - gap
        if accuracy < prec:
            if options.get('verbose'):
                print "SIN/COS", accuracy, "wanted", prec, "gap", gap
                print to_str(y,10)
            if xprec > options.get('maxprec', DEFAULT_MAXPREC):
                return y, None, accuracy, None
            xprec += gap
            re, im, re_accuracy, im_accuracy = evalf(arg, xprec, options)
            continue
        else:
            return y, None, prec, None

def evalf_log(expr, prec, options):
    arg = expr.args[0]
    workprec = prec+10
    xre, xim, xacc, _ = evalf(arg, workprec, options)

    if xim:
        # XXX: use get_abs etc instead
        re = evalf_log(C.log(C.abs(arg, evaluate=False), evaluate=False), prec, options)
        im = mpf_atan2(xim, xre or fzero, prec)
        return re[0], im, re[2], prec

    imaginary_term = (mpf_cmp(xre, fzero) < 0)

    re = mpf_log(mpf_abs(xre), prec, round_nearest)
    size = fastlog(re)
    if prec - size > workprec:
        # We actually need to compute 1+x accurately, not x
        arg = C.Add(S.NegativeOne,arg,evaluate=False)
        xre, xim, xre_acc, xim_acc = evalf_add(arg, prec, options)
        prec2 = workprec - fastlog(xre)
        re = mpf_log(mpf_add(xre, fone, prec2), prec, round_nearest)

    re_acc = prec

    if imaginary_term:
        return re, mpf_pi(prec), re_acc, prec
    else:
        return re, None, re_acc, None

def evalf_atan(v, prec, options):
    arg = v.args[0]
    xre, xim, reacc, imacc = evalf(arg, prec+5, options)
    if xim:
        raise NotImplementedError
    return mpf_atan(xre, prec, round_nearest), None, prec, None

def evalf_piecewise(expr, prec, options):
    if 'subs' in options:
        expr = expr.subs(options['subs'])
        del options['subs']
        if hasattr(expr,'func'):
            return evalf(expr, prec, options)
        if type(expr) == float:
            return evalf(C.Real(expr), prec, options)
        if type(expr) == int:
            return evalf(C.Integer(expr), prec, options)

    # We still have undefined symbols
    raise NotImplementedError

def evalf_piecewise(expr, prec, options):
    if 'subs' in options:
        expr = expr.subs(options['subs'])
        del options['subs']
        if hasattr(expr,'func'):
            return evalf(expr, prec, options)
        if type(expr) == float:
            return evalf(C.Real(expr), prec, options)
        if type(expr) == int:
            return evalf(C.Integer(expr), prec, options)

    # We still have undefined symbols
    raise NotImplementedError

def evalf_bernoulli(expr, prec, options):
    arg = expr.args[0]
    if not arg.is_Integer:
        raise ValueError("Bernoulli number index must be an integer")
    n = int(arg)
    b = mpf_bernoulli(n, prec, round_nearest)
    if b == fzero:
        return None, None, None, None
    return b, None, prec, None

#----------------------------------------------------------------------------#
#                                                                            #
#                            High-level operations                           #
#                                                                            #
#----------------------------------------------------------------------------#

def as_mpmath(x, prec, options):
    x = sympify(x)
    if isinstance(x, C.Zero):
        return mpf(0)
    if isinstance(x, C.Infinity):
        return mpf('inf')
    if isinstance(x, C.NegativeInfinity):
        return mpf('-inf')
    # XXX
    re, im, _, _ = evalf(x, prec, options)
    if im:
        return mpc(re or fzero, im)
    return mpf(re)

def do_integral(expr, prec, options):
    func = expr.args[0]
    x, (xlow, xhigh) = expr.args[1][0]
    orig = mp.prec

    oldmaxprec = options.get('maxprec', DEFAULT_MAXPREC)
    options['maxprec'] = min(oldmaxprec, 2*prec)

    try:
        mp.prec = prec+5
        xlow = as_mpmath(xlow, prec+15, options)
        xhigh = as_mpmath(xhigh, prec+15, options)

        # Integration is like summation, and we can phone home from
        # the integrand function to update accuracy summation style
        # Note that this accuracy is inaccurate, since it fails
        # to account for the variable quadrature weights,
        # but it is better than nothing

        have_part = [False, False]
        max_real_term = [MINUS_INF]
        max_imag_term = [MINUS_INF]

        def f(t):
            re, im, re_acc, im_acc = evalf(func, mp.prec, {'subs':{x:t}})

            have_part[0] = re or have_part[0]
            have_part[1] = im or have_part[1]

            max_real_term[0] = max(max_real_term[0], fastlog(re))
            max_imag_term[0] = max(max_imag_term[0], fastlog(im))

            if im:
                return mpc(re or fzero, im)
            return mpf(re or fzero)

        if options.get('quad') == 'osc':
            A = C.Wild('A', exclude=[x])
            B = C.Wild('B', exclude=[x])
            D = C.Wild('D')
            m = func.match(C.cos(A*x+B)*D)
            if not m:
                m = func.match(C.sin(A*x+B)*D)
            if not m:
                raise ValueError("An integrand of the form sin(A*x+B)*f(x) "
                  "or cos(A*x+B)*f(x) is required for oscillatory quadrature")
            period = as_mpmath(2*S.Pi/m[A], prec+15, options)
            result = quadosc(f, [xlow, xhigh], period=period)
            # XXX: quadosc does not do error detection yet
            quadrature_error = MINUS_INF
        else:
            result, quadrature_error = quadts(f, [xlow, xhigh], error=1)
            quadrature_error = fastlog(quadrature_error._mpf_)

    finally:
        options['maxprec'] = oldmaxprec
        mp.prec = orig

    if have_part[0]:
        re = result.real._mpf_
        if re == fzero:
            re = mpf_shift(fone, min(-prec,-max_real_term[0],-quadrature_error))
            re_acc = -1
        else:
            re_acc = -max(max_real_term[0]-fastlog(re)-prec, quadrature_error)
    else:
        re, re_acc = None, None

    if have_part[1]:
        im = result.imag._mpf_
        if im == fzero:
            im = mpf_shift(fone, min(-prec,-max_imag_term[0],-quadrature_error))
            im_acc = -1
        else:
            im_acc = -max(max_imag_term[0]-fastlog(im)-prec, quadrature_error)
    else:
        im, im_acc = None, None

    result = re, im, re_acc, im_acc
    return result

def evalf_integral(expr, prec, options):
    workprec = prec
    i = 0
    maxprec = options.get('maxprec', INF)
    while 1:
        result = do_integral(expr, workprec, options)
        accuracy = complex_accuracy(result)
        if accuracy >= prec or workprec >= maxprec:
            return result
        workprec += prec - max(-2**i, accuracy)
        i += 1

def check_convergence(numer, denom, n):
    """
    Returns (h, g, p) where
    -- h is:
        > 0 for convergence of rate 1/factorial(n)**h
        < 0 for divergence of rate factorial(n)**(-h)
        = 0 for geometric or polynomial convergence or divergence

    -- abs(g) is:
        > 1 for geometric convergence of rate 1/h**n
        < 1 for geometric divergence of rate h**n
        = 1 for polynomial convergence or divergence

        (g < 0 indicates an alternating series)

    -- p is:
        > 1 for polynomial convergence of rate 1/n**h
        <= 1 for polynomial divergence of rate n**(-h)

    """
    npol = C.Poly(numer, n)
    dpol = C.Poly(denom, n)
    p = npol.degree
    q = dpol.degree
    rate = q - p
    if rate:
        return rate, None, None
    constant = dpol.lead_term[0] / npol.lead_term[0]
    if abs(constant) != 1:
        return rate, constant, None
    if npol.degree == dpol.degree == 0:
        return rate, constant, 0
    pc = list(npol.iter_all_terms())[1][0]
    qc = list(dpol.iter_all_terms())[1][0]
    return rate, constant, qc-pc

def hypsum(expr, n, start, prec):
    """
    Sum a rapidly convergent infinite hypergeometric series with
    given general term, e.g. e = hypsum(1/factorial(n), n). The
    quotient between successive terms must be a quotient of integer
    polynomials.
    """
    from sympy import hypersimp, lambdify

    if start:
        expr = expr.subs(n, n+start)
    hs = hypersimp(expr, n)
    if hs is None:
        raise NotImplementedError("a hypergeometric series is required")
    num, den = hs.as_numer_denom()

    func1 = lambdify(n, num)
    func2 = lambdify(n, den)

    h, g, p = check_convergence(num, den, n)

    if h < 0:
        raise ValueError("Sum diverges like (n!)^%i" % (-h))

    # Direct summation if geometric or faster
    if h > 0 or (h == 0 and abs(g) > 1):
        one = MP_BASE(1) << prec
        term = expr.subs(n, 0)
        term = (MP_BASE(term.p) << prec) // term.q
        s = term
        k = 1
        while abs(term) > 5:
            term *= MP_BASE(func1(k-1))
            term //= MP_BASE(func2(k-1))
            s += term
            k += 1
        return from_man_exp(s, -prec)
    else:
        alt = g < 0
        if abs(g) < 1:
            raise ValueError("Sum diverges like (%i)^n" % abs(1/g))
        if p < 1 or (p == 1 and not alt):
            raise ValueError("Sum diverges like n^%i" % (-p))
        # We have polynomial convergence: use Richardson extrapolation
        # Need to use at least quad precision because a lot of cancellation
        # might occur in the extrapolation process
        prec2 = 4*prec
        one = MP_BASE(1) << prec2
        term = expr.subs(n, 0)
        term = (MP_BASE(term.p) << prec2) // term.q

        def summand(k, _term=[term]):
            if k:
                k = int(k)
                _term[0] *= MP_BASE(func1(k-1))
                _term[0] //= MP_BASE(func2(k-1))
            return make_mpf(from_man_exp(_term[0], -prec2))

        orig = mp.prec
        try:
            mp.prec = prec
            v = nsum(summand, [0, mpmath_inf], method='richardson')
        finally:
            mp.prec = orig
        return v._mpf_

def evalf_sum(expr, prec, options):
    func = expr.function
    limits = expr.limits
    if len(limits) != 1 or not isinstance(limits[0], tuple) or \
        len(limits[0]) != 3:
        raise NotImplementedError
    prec2 = prec+10
    try:
        n, a, b = limits[0]
        if b != S.Infinity or a != int(a):
            raise NotImplementedError
        # Use fast hypergeometric summation if possible
        v = hypsum(func, n, int(a), prec2)
        delta = prec - fastlog(v)
        if fastlog(v) < -10:
            v = hypsum(func, n, int(a), delta)
        return v, None, min(prec, delta), None
    except NotImplementedError:
        # Euler-Maclaurin summation for general series
        eps = C.Real(2.0)**(-prec)
        for i in range(1, 5):
            m = n = 2**i * prec
            s, err = expr.euler_maclaurin(m=m, n=n, eps=eps, \
                eval_integral=False)
            err = err.evalf()
            if err <= eps:
                break
        err = fastlog(evalf(abs(err), 20, options)[0])
        re, im, re_acc, im_acc = evalf(s, prec2, options)
        re_acc = max(re_acc, -err)
        im_acc = max(im_acc, -err)
        return re, im, re_acc, im_acc


#----------------------------------------------------------------------------#
#                                                                            #
#                            Symbolic interface                              #
#                                                                            #
#----------------------------------------------------------------------------#

def evalf_symbol(x, prec, options):
    val = options['subs'][x]
    if isinstance(val, mpf):
        if not val:
            return None, None, None, None
        return val._mpf_, None, prec, None
    else:
        if not '_cache' in options:
            options['_cache'] = {}
        cache = options['_cache']
        cached, cached_prec = cache.get(x.name, (None, MINUS_INF))
        if cached_prec >= prec:
            return cached
        v = evalf(sympify(val), prec, options)
        cache[x.name] = (v, prec)
        return v

evalf_table = None

def _create_evalf_table():
    global evalf_table
    evalf_table = {
    C.Symbol : evalf_symbol,
    C.Dummy : evalf_symbol,
    C.Real : lambda x, prec, options: (x._mpf_, None, prec, None),
    C.Rational : lambda x, prec, options: (from_rational(x.p, x.q, prec), None, prec, None),
    C.Integer : lambda x, prec, options: (from_int(x.p, prec), None, prec, None),
    C.Zero : lambda x, prec, options: (None, None, prec, None),
    C.One : lambda x, prec, options: (fone, None, prec, None),
    C.Half : lambda x, prec, options: (fhalf, None, prec, None),
    C.Pi : lambda x, prec, options: (mpf_pi(prec), None, prec, None),
    C.Exp1 : lambda x, prec, options: (mpf_e(prec), None, prec, None),
    C.ImaginaryUnit : lambda x, prec, options: (None, fone, None, prec),
    C.NegativeOne : lambda x, prec, options: (fnone, None, prec, None),

    C.exp : lambda x, prec, options: evalf_pow(C.Pow(S.Exp1, x.args[0],
        evaluate=False), prec, options),

    C.cos : evalf_trig,
    C.sin : evalf_trig,

    C.Add : evalf_add,
    C.Mul : evalf_mul,
    C.Pow : evalf_pow,

    C.log : evalf_log,
    C.atan : evalf_atan,
    C.abs : evalf_abs,

    C.re : evalf_re,
    C.im : evalf_im,
    C.floor : evalf_floor,
    C.ceiling : evalf_ceiling,

    C.Integral : evalf_integral,
    C.Sum : evalf_sum,
    C.Piecewise : evalf_piecewise,

    C.bernoulli : evalf_bernoulli,
    }

def evalf(x, prec, options):
    try:
        rf = evalf_table[x.func]
        r = rf(x, prec, options)
    except KeyError:
        #r = finalize_complex(x._eval_evalf(prec)._mpf_, fzero, prec)
        try:
            # Fall back to ordinary evalf if possible
            if 'subs' in options:
                x = x.subs(options['subs'])
            r = x._eval_evalf(prec)._mpf_, None, prec, None
        except AttributeError:
            raise NotImplementedError
    if options.get("verbose"):
        print "### input", x
        print "### output", to_str(r[0] or fzero, 50)
        #print "### raw", r[0], r[2]
        print
    if options.get("chop"):
        r = chop_parts(r, prec)
    if options.get("strict"):
        check_target(x, r, prec)
    return r

def Basic_evalf(x, n=15, **options):
    """
    Evaluate the given formula to an accuracy of n digits.
    Optional keyword arguments:

        subs=<dict>
            Substitute numerical values for symbols, e.g.
            subs={x:3, y:1+pi}.

        maxprec=N
            Allow a maximum temporary working precision of N digits
            (default=100)

        chop=<bool>
            Replace tiny real or imaginary parts in subresults
            by exact zeros (default=False)

        strict=<bool>
            Raise PrecisionExhausted if any subresult fails to evaluate
            to full accuracy, given the available maxprec
            (default=False)

        quad=<str>
            Choose algorithm for numerical quadrature. By default,
            tanh-sinh quadrature is used. For oscillatory
            integrals on an infinite interval, try quad='osc'.

        verbose=<bool>
            Print debug information (default=False)

    """
    if not evalf_table:
        _create_evalf_table()
    prec = dps_to_prec(n)
    if 'maxprec' in options:
        options['maxprec'] = int(options['maxprec']*LG10)
    else:
        options['maxprec'] = max(prec, DEFAULT_MAXPREC)
    try:
        result = evalf(x, prec+4, options)
    except NotImplementedError:
        # Fall back to the ordinary evalf
        v = x._eval_evalf(prec)
        if v is None:
            return x
        try:
            # If the result is numerical, normalize it
            result = evalf(v, prec, options)
        except:
            # Probably contains symbols or unknown functions
            return v
    re, im, re_acc, im_acc = result
    if re:
        p = max(min(prec, re_acc), 1)
        #re = fpos(re, p, round_nearest)
        re = C.Real._new(re, p)
    else:
        re = S.Zero
    if im:
        p = max(min(prec, im_acc), 1)
        #im = fpos(im, p, round_nearest)
        im = C.Real._new(im, p)
        return re + im*S.ImaginaryUnit
    else:
        return re

Basic.evalf = Basic.n = Basic_evalf

def N(x, n=15, **options):
    """
    Calls x.evalf(n, **options).

    Both .evalf() and N() are equivalent, use the one that you like better.

    Example:
    >>> from sympy import Sum, Symbol, oo
    >>> k = Symbol("k")
    >>> Sum(1/k**k, (k, 1, oo))
    Sum(k**(-k), (k, 1, oo))
    >>> N(Sum(1/k**k, (k, 1, oo)), 4)
    1.291

    """
    return sympify(x).evalf(n, **options)



from sympy import Lambda, Symbol, Function, WildFunction, Derivative, sqrt, \
        log, exp, Rational, Real, sign, Basic, sin, cos, diff, I, re, im, \
        oo, zoo, nan, E, expand, pi
from sympy.utilities.pytest import XFAIL
from sympy.abc import x, y


def test_log():
    assert log(2) > 0
    assert log(1).is_zero
    assert log(0.5).is_negative == True

def test_exp_log():
    x = Symbol("x", real=True)
    assert log(exp(x)) == x
    assert exp(log(x)) == x

def test_log_expansion():
    x = Symbol("x", real=True)
    y = Symbol("y", real=True)

    # ok in interactive, fails in py.test
    #assert log(x*y) != log(x)+log(y)
    #assert log(x**2) != 2*log(x)

    assert log(x*y).expand() == log(x)+log(y)
    assert log(x**2).expand() == 2*log(x)
    assert (log(x**-5)**-1).expand() == -1/log(x)/5

def test_log_hashing_bug():
    x = Symbol("y")
    assert x != log(log(x))
    assert hash(x) != hash(log(log(x)))
    assert log(x) != log(log(log(x)))

    e = 1/log(log(x)+log(log(x)))
    assert e.base.func is log
    e = 1/log(log(x)+log(log(log(x))))
    assert e.base.func is log

    x = Symbol("x")
    e = log(log(x))
    assert e.func is log
    assert not x.func is log
    assert hash(log(log(x))) != hash(x)
    assert e != x

def test_sign():
    assert sign(log(2)) == 1

def test_exp_bug():
    x = Symbol("x")
    assert exp(1*log(x)) == x

def test_exp_expand():
    x = Symbol("x")
    y = Symbol("y")

    e = exp(log(Rational(2))*(1+x)-log(Rational(2))*x)
    assert e.expand() == 2
    # XXX The RHS is automatically combined so that it equals the LHS
    #assert exp(x+y) != exp(x)*exp(y)
    assert exp(x+y).expand() == exp(x)*exp(y)


def test_f_expand_complex():
    f = Function('f')
    x = Symbol('x', real=True)
    z = Symbol('z')

    assert f(x).expand(complex=True)        == I*im(f(x)) + re(f(x))
    assert exp(x).expand(complex=True)      == exp(x)
    assert exp(I*x).expand(complex=True)    == cos(x) + I*sin(x)
    assert exp(z).expand(complex=True)      == cos(im(z))*exp(re(z)) + \
                                             I*sin(im(z))*exp(re(z))

def test_bug1():
    x = Symbol("x")
    w = Symbol("w")

    e = sqrt(-log(w))
    assert e.subs(log(w),-x) == sqrt(x)

    e = sqrt(-5*log(w))
    assert e.subs(log(w),-x) == sqrt(5*x)

def test_general_function():
    nu = Function('nu', nargs=1)
    x = Symbol("x")
    y = Symbol("y")

    e = nu(x)
    edx = e.diff(x)
    edy = e.diff(y)
    edxdx = e.diff(x).diff(x)
    edxdy = e.diff(x).diff(y)
    assert e == nu(x)
    assert edx != nu(x)
    assert edx == diff(nu(x), x)
    assert edy == 0
    assert edxdx == diff(diff(nu(x), x), x)
    assert edxdy == 0

def test_function_nargs():
    f = Function('f')
    x = Symbol('x')
    assert f.nargs == None
    assert f(x).nargs == 1
    assert f(x, x, x, x).nargs == 4

def test_derivative_subs_bug():
    x = Symbol("x")
    l = Function('l', nargs=1)
    n = Function('n', nargs=1)

    e = diff(n(x), x)
    assert e.subs(n(x), l(x)) != e
    assert e.subs(n(x), l(x)) == diff(l(x), x)
    assert e.subs(n(x), -l(x)) == diff(-l(x), x)

def test_derivative_subs_self_bug():
    f = Function('f')
    d = diff(f(x), x)

    assert d.subs(d, y) == y


def test_derivative_linearity():
    x = Symbol("x")
    y = Symbol("y")
    n = Function('n', nargs=1)

    assert diff(-n(x), x) == -diff(n(x), x)
    assert diff(8*n(x), x) == 8*diff(n(x), x)
    assert diff(8*n(x), x) != 7*diff(n(x), x)
    assert diff(8*n(x)*x, x) == 8*n(x) + 8*x*diff(n(x), x)
    assert diff(8*n(x)*y*x, x) == 8*y*n(x) + 8*y*x*diff(n(x), x)

def test_derivative_evaluate():
    x = Symbol('x')

    assert Derivative(sin(x), x) != diff(sin(x), x)
    assert Derivative(sin(x), x).doit() == diff(sin(x), x)

    f = Function('f')
    assert Derivative(Derivative(f(x), x), x) == diff(f(x), x, x)

@XFAIL
def test_combine():
    # XXX combine no longer exists
    x = Symbol("x")
    y = Symbol("y")
    assert exp(x)*exp(-x) != 1
    assert (exp(x)*exp(-x)).combine() == 1

    assert exp(x)**2 != exp(2*x)
    assert (exp(x)**2).combine() == exp(2*x)

    assert exp(x)*exp(-x/2)*exp(-x/2) != 1
    assert (exp(x)*exp(-x/2)*exp(-x/2)).combine() == 1

    assert (2*log(x)).combine() == log(x**2)
    assert exp(2*log(x)) != x**2
    assert exp(2*log(x)).combine() == x**2

    assert exp(x)*exp(-x)-1 !=0
    assert (exp(x)*exp(-x)-1).combine() == 0

    assert (2*exp(x)*exp(-x)).combine() == 2
    assert (x/exp(x)*exp(-x)).combine() == x*exp(-2*x)

def test_Lambda():
    e = Lambda(x, x**2)
    f = Function('f')
    assert e(4) == 16
    assert e(x) == x**2
    assert e(y) == y**2

    assert Lambda(x, x**2) == Lambda(x, x**2)
    assert Lambda(x, x**2) == Lambda(y, y**2)
    assert Lambda(x, x**2) != Lambda(y, y**2+1)
    assert Lambda(x,y,x**y) == Lambda(y,x,y**x)
    assert Lambda(x,y,x**y) != Lambda(x,y,y**x)

    assert Lambda(x,y,x**y)(x,y) == x**y
    assert Lambda(x,y,x**y)(x) == Lambda(y,x**y)
    assert Lambda(x,y,x**y)(x)(y) == x**y
    assert Lambda(x,y,x**y)(x)(3) == x**3
    assert Lambda(x,y,x**y)(3)(y) == 3**y
    assert Lambda(x,y,x**y)(3)(3) == 3**3
    assert Lambda(x,y,x**y)(3,3) == 3**3
    assert Lambda(x,y,x**y)(x,3) == x**3
    assert Lambda(x,y,x**y)(3,y) == 3**y
    assert Lambda(x,f(x))(x) == f(x)
    assert Lambda(x,f(x))() == Lambda(x,f(x))
    assert Lambda(x,x**2)(e(x)) == x**4
    assert e(e(x)) == x**4
    assert Lambda(x,y,f(x)+f(y))(x) == Lambda(y,f(x)+f(y))
    #doesn't work yet:
    #class F(Function):
    #    pass
    #assert Lambda(x, F(x)) == F

    assert Lambda(x, y, x+y).nargs == 2

    z = Symbol('z')
    t = Symbol('t')
    p = x, y, z, t
    assert Lambda(p, t*(x+y+z))(*p) == t * (x + y + z)


def test_expand_function():
    assert expand(x+y) == x + y
    assert expand(x+y, complex=True) == I*im(x) + I*im(y) + re(x) + re(y)


def test_function_comparable():
    x = Symbol('x')

    assert sin(x).is_comparable == False
    assert cos(x).is_comparable == False

    assert sin(Real('0.1')).is_comparable   == True
    assert cos(Real('0.1')).is_comparable   == True

    assert sin(E).is_comparable     == True
    assert cos(E).is_comparable     == True

    assert sin(Rational(1,3)).is_comparable == True
    assert cos(Rational(1,3)).is_comparable == True

@XFAIL
def test_function_comparable_fail():
    x = Symbol('x')

    assert sin(oo).is_comparable    == False
    assert sin(-oo).is_comparable   == False
    assert sin(zoo).is_comparable   == False
    assert sin(nan).is_comparable   == False

def test_deriv1():
    f=Function('f')
    g=Function('g')
    x = Symbol('x')
    assert f(g(x)).diff(x) == Derivative(f(g(x)), g(x)) * Derivative(g(x), x)

def test_deriv2():
    f=Function('f')
    g=Function('g')
    x = Symbol('x')

    assert f(x).diff(x) == Derivative(f(x), x)
    assert f(2*x).diff(x) == 2*Derivative(f(2*x), 2*x)
    assert (f(x)**3).diff(x) == 3*f(x)**2*f(x).diff(x)
    assert (f(2*x)**3).diff(x) == 6*f(2*x)**2*Derivative(f(2*x), 2*x)

    assert f(2+x).diff(x) == Derivative(f(2+x), 2+x)
    assert f(2+3*x).diff(x) == 3*Derivative(f(2+3*x), 2+3*x)
    assert f(sin(x)).diff(x) == Derivative(f(sin(x)), sin(x)) * cos(x)
    assert f(3*sin(x)).diff(x) == 3*Derivative(f(3*sin(x)), 3*sin(x)) * cos(x)

def test_deriv3():
    f=Function('f')
    g=Function('g')
    x = Symbol('x')

    assert (x**3).diff(x) == 3*x**2
    assert (x**3).diff(x, evaluate=False) != 3*x**2
    assert (x**3).diff(x, evaluate=False) == Derivative(x**3, x)

    assert diff(x**3, x) == 3*x**2
    assert diff(x**3, x, evaluate=False) != 3*x**2
    assert diff(x**3, x, evaluate=False) == Derivative(x**3, x)

def test_suppressed_evaluation():
    a = sin(0, evaluate=False)
    assert a != 0
    assert a.func is sin
    assert a.args == (0,)

def test_function_evalf():
    def eq(a,b,eps):
        return abs(a-b) < eps
    assert eq(sin(1).evalf(15), Real("0.841470984807897"), 1e-13)
    assert eq(sin(2).evalf(25), Real("0.9092974268256816953960199",25), 1e-23)
    assert eq(sin(1+I).evalf(15), Real("1.29845758141598") + Real("0.634963914784736")*I, 1e-13)
    assert eq(exp(1+I).evalf(15), Real("1.46869393991588") + Real("2.28735528717884239")*I, 1e-13)
    assert eq(log(pi+sqrt(2)*I).evalf(15), Real("1.23699044022052") + Real("0.422985442737893")*I, 1e-13)
    assert eq(cos(100).evalf(15), Real("0.86231887228768"), 1e-13)



from sympy import Symbol, Wild, sin, cos, exp, sqrt, pi, Function, Derivative,\
        abc, Integer, Eq, symbols, Add, I, Real, log, Rational, Lambda, atan2

def test_subs():
    n3=Rational(3)
    n2=Rational(2)
    n6=Rational(6)
    x=Symbol("x")
    c=Symbol("c")
    e=x
    e=e.subs(x,n3)
    assert e == Rational(3)

    e=2*x
    assert e == 2*x
    e=e.subs(x,n3)
    assert e == Rational(6)

    e=(sin(x)**2).diff(x)
    assert e == 2*sin(x)*cos(x)
    e=e.subs(x,n3)
    assert e == 2*cos(n3)*sin(n3)

    e=(sin(x)**2).diff(x)
    assert e == 2*sin(x)*cos(x)
    e=e.subs(sin(x),cos(x))
    assert e == 2*cos(x)**2

    assert exp(pi).subs(exp, sin) == 0
    assert cos(exp(pi)).subs(exp, sin) == 1

def test_logexppow():   # no eval()
    x = Symbol("x")
    w = Symbol("dummy :)")
    e = (3**(1+x)+2**(1+x))/(3**x+2**x)
    assert e.subs(2**x, w) != e
    assert e.subs(exp(x*log(Rational(2))),w) != e

def test_bug():
    x1=Symbol("x1")
    x2=Symbol("x2")
    y=x1*x2
    y.subs(x1,Real(3.0))

def test_subbug1():
    x=Symbol("x")
    e=(x**x).subs(x,1)
    e=(x**x).subs(x,1.0)

def test_subbug2():
    # Ensure this does not cause infinite recursion
    x = Symbol('x')
    assert Real(7.7).epsilon_eq(abs(x).subs(x, -7.7))

def test_dict():
    x = Symbol('x')
    a,b,c = map(Wild, 'abc')

    f = 3*cos(4*x)
    r = f.match(a*cos(b*x))
    assert r == {a: 3, b: 4}
    e =  a/b * sin(b*x)
    assert e._subs_dict(r) == r[a]/r[b] * sin(r[b]*x)
    assert e._subs_dict(r) == 3 * sin(4*x) / 4


def test_dict_ambigous():   # see #467
    x = Symbol('x')
    y = Symbol('y')
    z = Symbol('z')

    f = x*exp(x)
    g = z*exp(z)

    df= {x:y, exp(x): y}
    dg= {z:y, exp(z): y}

    assert f._subs_dict(df) == y**2
    assert g._subs_dict(dg) == y**2

    # and this is how order can affect the result
    assert f .subs(x,y) .subs(exp(x),y)  == y*exp(y)
    assert f .subs(exp(x),y) .subs(x,y)  == y**2


def test_deriv_sub_bug3():
    x = Symbol("x")
    y = Symbol("y")
    f = Function("f")
    pat = Derivative(f(x), x, x)
    assert pat.subs(y, y**2) == Derivative(f(x), x, x)
    assert pat.subs(y, y**2) != Derivative(f(x), x)

def test_equality_subs1():
    f = Function("f")
    x = abc.x
    eq = Eq(f(x)**2, x)
    res = Eq(Integer(16), x)
    assert eq.subs(f(x), 4) == res

def test_equality_subs2():
    f = Function("f")
    x = abc.x
    eq = Eq(f(x)**2, 16)
    assert bool(eq.subs(f(x), 3)) == False
    assert bool(eq.subs(f(x), 4)) == True

def test_issue643():
    x = Symbol('x')
    y = Symbol('y')

    e = sqrt(x)*exp(y)
    assert e.subs(sqrt(x), 1)   == exp(y)

def test_subs_dict1():
    x, y = symbols('xy')
    assert (1+x*y).subs(x, pi) == 1 + pi*y
    assert (1+x*y).subs({x:pi, y:2}) == 1 + 2*pi

def test_subs_dict2():
    x = Symbol('x')
    a,b,c = map(Wild, 'abc')

    f = 3*cos(4*x)
    r = f.match(a*cos(b*x))
    assert r == {a: 3, b: 4}
    e =  a/b * sin(b*x)
    assert e.subs(r) == r[a]/r[b] * sin(r[b]*x)
    assert e.subs(r) == 3 * sin(4*x) / 4

def test_mul():
    x, y, z = map(Symbol, 'xyz')
    assert (x*y*z).subs(z*x,y) == y**2
    assert (2*x*y).subs(5*x*y,z) == 2*z/5

def test_add():
    a, b, c, d, x = abc.a, abc.b, abc.c, abc.d, abc.x
    assert (a**2 - b - c).subs(a**2 - b, d) in [d - c, a**2 - b - c]
    assert (a**2 - c).subs(a**2 - c, d) == d
    assert (a**2 - b - c).subs(a**2 - c, d) in [d - b, a**2 - b - c]
    assert (a**2 - x - c).subs(a**2 - c, d) in [d - x, a**2 - x - c]
    assert (a**2 - b - sqrt(a)).subs(a**2 - sqrt(a), c) == c - b
    assert (a+b+exp(a+b)).subs(a+b,c) == c + exp(c)
    assert (c+b+exp(c+b)).subs(c+b,a) == a + exp(a)

    # this should work everytime:
    e = a**2 - b - c
    assert e.subs(Add(*e.args[:2]), d) == d + e.args[2]
    assert e.subs(a**2 - c, d) == d - b

def test_subs_issue910():
    assert (I*Symbol("a")).subs(1, 2) == I*Symbol("a")

def test_subs_subs_nums():
    x = Symbol("x")
    assert sin(1).subs(1, 2) == sin(2)
    assert sin(2).subs(1, 3) == sin(2)
    assert (2*x).subs(1, 3) == 2*x
    assert (2*x).subs(2, 3) == 3*x
    assert (2*x).subs(x, 3) == 6

def test_functions_subs():
    x, y = map(Symbol, 'xy')
    f, g = map(Function, 'fg')
    l = Lambda(x, y, sin(x) + y)
    assert (g(y, x)+cos(x)).subs(g, l) == sin(y) + x + cos(x)
    assert (f(x)**2).subs(f, sin) == sin(x)**2
    assert (f(x,y)).subs(f,log) == log(x,y)
    assert (f(x,y)).subs(f,sin) == f(x,y)
    assert (sin(x)+atan2(x,y)).subs([[atan2,f],[sin,g]]) == f(x,y) + g(x)
    assert (g(f(x+y, x))).subs([[f, l], [g, exp]]) == exp(x + sin(x + y))



# -*- coding: utf-8 -*-

import sympy
from repr import ReprPrinter
from str import StrPrinter

# A list of classes that should be printed using StrPrinter
STRPRINT = ("Add", "Infinity", "Integer", "Mul", "NegativeInfinity",
            "NegativeOne", "One", "Pow", "Zero")

class PythonPrinter(ReprPrinter, StrPrinter):
    """A printer which converts an expression into its Python interpretation."""

    def __init__(self):
        ReprPrinter.__init__(self)
        StrPrinter.__init__(self)
        self.symbols = []
        self.functions = []

        # Create print methods for classes that should use StrPrinter instead
        # of ReprPrinter.
        for name in STRPRINT:
            f_name = "_print_%s"%name
            f = getattr(StrPrinter, f_name)
            setattr(PythonPrinter, f_name, f)

    def _print_Function(self, expr):
        func = expr.func.__name__
        if not hasattr(sympy, func) and not func in self.functions:
            self.functions.append(func)
        return StrPrinter._print_Function(self, expr)

    # procedure (!) for defining symbols which have be defined in print_python()
    def _print_Symbol(self, expr):
        symbol = self._str(expr)
        if symbol not in self.symbols:
            self.symbols.append(symbol)
        return StrPrinter._print_Symbol(self, expr)

    def _print_module(self, expr):
        raise Exception('Modules in the expression are unacceptable')


def python(expr):
    """Return Python interpretation of passed expression
    (can be passed to the exec() function without any modifications)"""

    printer = PythonPrinter()
    expr = printer.doprint(expr)

    result = ''
    # Returning found symbols and functions
    for symbol in printer.symbols:
        result += symbol + ' = Symbol(\'' + symbol + '\')\n'
    for function in printer.functions:
        result += function + ' = Function(\'' + function + '\')\n'

    result += 'e = ' + printer._str(expr)
    return result

def print_python(expr):
    """Print output of python() function"""
    print python(expr)




'''Fallback driver producing no audio.
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: silent.py 1322 2007-10-23 12:58:03Z Alex.Holkner $'

import time

from pyglet.media import AudioPlayer, Listener, AudioData
from pyglet.media import MediaException

class SilentAudioPlayer(AudioPlayer):
    def __init__(self, audio_format):
        super(SilentAudioPlayer, self).__init__(audio_format)

        self._playing = False
        self._eos_count = 0

        self._audio_data_list = []
        self._head_time = 0.0
        self._head_system_time = time.time()

    def get_write_size(self):
        return max(0, 10000 - sum(
            [a.length for a in self._audio_data_list if a is not None]))

    def write(self, audio_data):
        if not self._audio_data_list:
            self._head_time = 0.0
            self._head_system_time = time.time()
        self._audio_data_list.append(
            AudioData(None, 
                      audio_data.length, 
                      audio_data.timestamp,
                      audio_data.duration))
        audio_data.consume(audio_data.length, self.audio_format)

    def write_eos(self):
        if self._audio_data_list:
            self._audio_data_list.append(None)

    def write_end(self):
        pass

    def play(self):
        self._playing = True
        self._head_system_time = time.time()

    def stop(self):
        self._playing = False
        self._head_time = time.time() - self._head_system_time

    def clear(self):
        self._audio_data_list = []
        self._head_time = 0.0
        self._head_system_time = time.time()
        self._eos_count = 0

    def pump(self):
        if not self._playing:
            return
        system_time = time.time()
        head_time = system_time - self._head_system_time
        try:
            while head_time >= self._audio_data_list[0].duration:
                head_time -= self._audio_data_list[0].duration
                self._audio_data_list.pop(0)
                while self._audio_data_list[0] is None:
                    self._eos_count += 1
                    self._audio_data_list.pop(0)
            self._head_system_time = system_time - head_time
            return head_time + self._audio_data_list[0].timestamp
        except IndexError:
            return 0.0

    def get_time(self):
        if not self._audio_data_list:
            return 0.0

        if self._playing:
            system_time = time.time()
            head_time = system_time - self._head_system_time
            return head_time + self._audio_data_list[0].timestamp 
        else:
            return self._audio_data_list[0].timestamp + self._head_time

    def clear_eos(self):
        if self._eos_count:
            self._eos_count -= 1 
            return True
        return False

class SilentListener(Listener):
    def _set_volume(self, volume):
        self._volume = volume

    def _set_position(self, position):
        self._position = position

    def _set_velocity(self, velocity):
        self._velocity = velocity

    def _set_forward_orientation(self, orientation):
        self._forward_orientation = orientation

    def _set_up_orientation(self, orientation):
        self._up_orientation = orientation

    def _set_doppler_factor(self, factor):
        self._doppler_factor = factor

    def _set_speed_of_sound(self, speed_of_sound):
        self._speed_of_sound = speed_of_sound

def driver_init():
    pass

driver_listener = SilentListener()
driver_audio_player_class = SilentAudioPlayer



from sympy import Symbol, Wild, Inequality, StrictInequality, pi, I, Rational, \
        sympify, raises, symbols


def test_Symbol():
    a = Symbol("a")
    x1 = Symbol("x")
    x2 = Symbol("x")
    xdummy1 = Symbol("x", dummy=True)
    xdummy2 = Symbol("x", dummy=True)

    assert a != x1
    assert a != x2
    assert x1 == x2
    assert x1 != xdummy1
    assert xdummy1 != xdummy2

    assert Symbol("x") == Symbol("x")
    assert Symbol("x", dummy=True) != Symbol("x", dummy=True)


def test_lt_gt():
    x, y = Symbol('x'), Symbol('y')

    assert (x <= y) == Inequality(x, y)
    assert (x >= y) == Inequality(y, x)
    assert (x <= 0) == Inequality(x, 0)
    assert (x >= 0) == Inequality(0, x)

    assert (x < y) == StrictInequality(x, y)
    assert (x > y) == StrictInequality(y, x)
    assert (x < 0) == StrictInequality(x, 0)
    assert (x > 0) == StrictInequality(0, x)

    assert (x**2+4*x+1 > 0) == StrictInequality(0, x**2+4*x+1)

def test_no_len():
    # there should be no len for numbers
    x = Symbol('x')
    xxl = Symbol('xxl')
    raises(TypeError, "len(x)")
    raises(TypeError, "len(xxl)")

def test_Wild_properties():
    # these tests only include Atoms
    x   = Symbol("x")
    y   = Symbol("y")
    p   = Symbol("p", positive=True)
    k   = Symbol("k", integer=True)
    r   = Symbol("r", real=True)
    n   = Symbol("n", integer=True, positive=True)

    given_patterns = [ x, y, p, k, -k, n, -n, sympify(-3), sympify(3), pi, Rational(3,2), I ]

    integerp = lambda k : k.is_integer
    positivep = lambda k : k.is_positive
    symbolp = lambda k : k.is_Symbol
    realp = lambda k : k.is_real

    S = Wild("S", properties=[symbolp])
    R = Wild("R", properties=[realp])
    Y = Wild("Y", exclude=[x,p,k,n])
    P = Wild("P", properties=[positivep])
    K = Wild("K", properties=[integerp])
    N = Wild("N", properties=[positivep, integerp])

    given_wildcards = [ S, R, Y, P, K, N ]

    goodmatch = {
        S : (x,y,p,k,n),
        R : (p,k,-k,n,-n,-3,3,pi,Rational(3,2)),
        Y : (y,-3,3,pi,Rational(3,2),I ),
        P : (p, n,3,pi, Rational(3,2)),
        K : (k,-k,n,-n,-3,3),
        N : (n,3)}

    for A in given_wildcards:
        for pat in given_patterns:
                d = pat.match(A)
                if pat in goodmatch[A]:
                    assert d[A] in goodmatch[A]
                else:
                    assert d == None

def test_symbols():
    x, y, z = Symbol('x'), Symbol('y'), Symbol('z')
    assert symbols('x') == Symbol('x')
    assert symbols('xyz') == [x, y, z]
    assert symbols('x y z') == symbols('x,y,z') == (x, y, z)
    assert symbols('xyz', each_char=False) == Symbol('xyz')
    x, y = symbols('x y', each_char=False, real=True)
    assert x.is_real and y.is_real




from sympy import bernoulli, Symbol, harmonic, Rational, oo, zoo, pi, bell, \
        fibonacci, lucas

x = Symbol('x')

def test_bernoulli():
    assert bernoulli(0) == 1
    assert bernoulli(1) == Rational(-1,2)
    assert bernoulli(2) == Rational(1,6)
    assert bernoulli(3) == 0
    assert bernoulli(4) == Rational(-1,30)
    assert bernoulli(5) == 0
    assert bernoulli(6) == Rational(1,42)
    assert bernoulli(7) == 0
    assert bernoulli(8) == Rational(-1,30)
    assert bernoulli(10) == Rational(5,66)
    assert bernoulli(1000001) == 0

    assert bernoulli(0, x) == 1
    assert bernoulli(1, x) == x-Rational(1,2)
    assert bernoulli(2, x) == x**2-x+Rational(1,6)
    assert bernoulli(3, x) == x**3 - (3*x**2)/2 + x/2

    # Should be fast; computed with mpmath
    b = bernoulli(1000)
    assert b.p % 10**10  == 7950421099
    assert b.q == 342999030

    b = bernoulli(10**6, evaluate=False).evalf()
    assert str(b) == '-2.23799235765713e+4767529'


def test_fibonacci():
    assert [fibonacci(n) for n in range(-3, 5)] == [2, -1, 1, 0, 1, 1, 2, 3]
    assert fibonacci(100) == 354224848179261915075
    assert [lucas(n) for n in range(-3, 5)] == [-4, 3, -1, 2, 1, 3, 4, 7]
    assert lucas(100) == 792070839848372253127

    assert fibonacci(1, x) == 1
    assert fibonacci(2, x) == x
    assert fibonacci(3, x) == x**2 + 1
    assert fibonacci(4, x) == x**3 + 2*x

def test_bell():
    assert [bell(n) for n in range(8)] == [1, 1, 2, 5, 15, 52, 203, 877]

    assert bell(0, x) == 1
    assert bell(1, x) == x
    assert bell(2, x) == x**2 + x
    assert bell(5, x) == x**5 + 10*x**4 + 25*x**3 + 15*x**2 + x

def test_harmonic():
    assert harmonic(1,1) == 1
    assert harmonic(2,1) == Rational(3,2)
    assert harmonic(3,1) == Rational(11,6)
    assert harmonic(4,1) == Rational(25,12)
    # assert harmonic(3,1) == harmonic(3)
    assert harmonic(3,5) == 1 + Rational(1,2**5) + Rational(1,3**5)
    assert harmonic(10,0) == 10
    assert harmonic(oo,1) == zoo
    assert harmonic(oo,2) == (pi**2)/6



# ----------------------------------------------------------------------------
# pyglet
# Copyright (c) 2006-2007 Alex Holkner
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions 
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright 
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#  * Neither the name of the pyglet nor the names of its
#    contributors may be used to endorse or promote products
#    derived from this software without specific prior written
#    permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------------

'''
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: freetype.py 1564 2007-12-31 23:57:48Z Alex.Holkner $'

import ctypes
from ctypes import *
from warnings import warn

import pyglet.lib
from pyglet.font import base
from pyglet import image
from pyglet.font.freetype_lib import *

# fontconfig library definitions
fontconfig = pyglet.lib.load_library('fontconfig')

FcResult = c_int

fontconfig.FcPatternBuild.restype = c_void_p
fontconfig.FcFontMatch.restype = c_void_p
fontconfig.FcFreeTypeCharIndex.restype = c_uint

FC_FAMILY = 'family'
FC_SIZE = 'size'
FC_SLANT = 'slant'
FC_WEIGHT = 'weight'
FC_FT_FACE = 'ftface'
FC_FILE = 'file'

FC_WEIGHT_REGULAR = 80
FC_WEIGHT_BOLD = 200

FC_SLANT_ROMAN = 0
FC_SLANT_ITALIC = 100

FT_STYLE_FLAG_ITALIC = 1
FT_STYLE_FLAG_BOLD = 2

(FT_RENDER_MODE_NORMAL,
 FT_RENDER_MODE_LIGHT,
 FT_RENDER_MODE_MONO,
 FT_RENDER_MODE_LCD,
 FT_RENDER_MODE_LCD_V) = range(5)

def FT_LOAD_TARGET_(x):
    return (x & 15) << 16

FT_LOAD_TARGET_NORMAL = FT_LOAD_TARGET_(FT_RENDER_MODE_NORMAL)
FT_LOAD_TARGET_LIGHT = FT_LOAD_TARGET_(FT_RENDER_MODE_LIGHT)
FT_LOAD_TARGET_MONO = FT_LOAD_TARGET_(FT_RENDER_MODE_MONO)
FT_LOAD_TARGET_LCD = FT_LOAD_TARGET_(FT_RENDER_MODE_LCD)
FT_LOAD_TARGET_LCD_V = FT_LOAD_TARGET_(FT_RENDER_MODE_LCD_V)

(FT_PIXEL_MODE_NONE,
 FT_PIXEL_MODE_MONO,
 FT_PIXEL_MODE_GRAY,
 FT_PIXEL_MODE_GRAY2,
 FT_PIXEL_MODE_GRAY4,
 FT_PIXEL_MODE_LCD,
 FT_PIXEL_MODE_LCD_V) = range(7)

(FcTypeVoid,
 FcTypeInteger,
 FcTypeDouble, 
 FcTypeString, 
 FcTypeBool,
 FcTypeMatrix,
 FcTypeCharSet,
 FcTypeFTFace,
 FcTypeLangSet) = range(9)
FcType = c_int

(FcMatchPattern,
 FcMatchFont) = range(2)
FcMatchKind = c_int

class _FcValueUnion(Union):
    _fields_ = [
        ('s', c_char_p),
        ('i', c_int),
        ('b', c_int),
        ('d', c_double),
        ('m', c_void_p),
        ('c', c_void_p),
        ('f', c_void_p),
        ('p', c_void_p),
        ('l', c_void_p),
    ]

class FcValue(Structure):
    _fields_ = [
        ('type', FcType),
        ('u', _FcValueUnion)
    ]

# End of library definitions

def f16p16_to_float(value):
    return float(value) / (1 << 16)

def float_to_f16p16(value):
    return int(value * (1 << 16))

def f26p6_to_float(value):
    return float(value) / (1 << 6)

def float_to_f26p6(value):
    return int(value * (1 << 6))

class FreeTypeGlyphRenderer(base.GlyphRenderer):
    def __init__(self, font):
        super(FreeTypeGlyphRenderer, self).__init__(font)
        self.font = font

    def render(self, text):
        face = self.font.face
        FT_Set_Char_Size(face, 0, self.font._face_size, 
                         self.font._dpi, self.font._dpi)
        glyph_index = fontconfig.FcFreeTypeCharIndex(byref(face), ord(text[0]))
        error = FT_Load_Glyph(face, glyph_index, FT_LOAD_RENDER)
        if error != 0:
            raise base.FontException(
                'Could not load glyph for "%c"' % text[0], error) 
        glyph_slot = face.glyph.contents
        width = glyph_slot.bitmap.width
        height = glyph_slot.bitmap.rows
        baseline = height - glyph_slot.bitmap_top
        lsb = glyph_slot.bitmap_left
        advance = int(f26p6_to_float(glyph_slot.advance.x))
        mode = glyph_slot.bitmap.pixel_mode
        pitch = glyph_slot.bitmap.pitch
        
        if mode == FT_PIXEL_MODE_MONO:
            # BCF fonts always render to 1 bit mono, regardless of render
            # flags. (freetype 2.3.5)
            bitmap_data = cast(glyph_slot.bitmap.buffer, 
                               POINTER(c_ubyte * (pitch * height))).contents
            data = (c_ubyte * (pitch * 8 * height))()
            data_i = 0
            for byte in bitmap_data:
                # Data is MSB; left-most pixel in a byte has value 128.
                data[data_i + 0] = (byte & 0x80) and 255 or 0
                data[data_i + 1] = (byte & 0x40) and 255 or 0
                data[data_i + 2] = (byte & 0x20) and 255 or 0
                data[data_i + 3] = (byte & 0x10) and 255 or 0
                data[data_i + 4] = (byte & 0x08) and 255 or 0
                data[data_i + 5] = (byte & 0x04) and 255 or 0
                data[data_i + 6] = (byte & 0x02) and 255 or 0
                data[data_i + 7] = (byte & 0x01) and 255 or 0
                data_i += 8
            pitch <<= 3
        elif mode == FT_PIXEL_MODE_GRAY:
            # Usual case
            data = glyph_slot.bitmap.buffer
        else:
            raise base.FontException('Unsupported render mode for this glyph')

        # pitch should be negative, but much faster to just swap tex_coords
        img = image.ImageData(width, height, 'A', data, pitch)
        glyph = self.font.create_glyph(img) 
        glyph.set_bearings(baseline, lsb, advance)
        t = list(glyph.tex_coords)
        glyph.tex_coords = t[9:12] + t[6:9] + t[3:6] + t[:3]

        return glyph

class FreeTypeMemoryFont(object):
    def __init__(self, data):
        self.buffer = (ctypes.c_byte * len(data))()
        ctypes.memmove(self.buffer, data, len(data))

        ft_library = ft_get_library()
        self.face = FT_Face()
        r = FT_New_Memory_Face(ft_library, 
            self.buffer, len(self.buffer), 0, self.face)
        if r != 0:
            raise base.FontException('Could not load font data')

        self.name = self.face.contents.family_name
        self.bold = self.face.contents.style_flags & FT_STYLE_FLAG_BOLD != 0
        self.italic = self.face.contents.style_flags & FT_STYLE_FLAG_ITALIC != 0

    def __del__(self):
        try:
            FT_Done_Face(self.face)
        except NameError:
            pass

class FreeTypeFont(base.Font):
    glyph_renderer_class = FreeTypeGlyphRenderer

    # Map font (name, bold, italic) to FreeTypeMemoryFont
    _memory_fonts = {}

    def __init__(self, name, size, bold=False, italic=False, dpi=None):
        super(FreeTypeFont, self).__init__()

        if dpi is None:
            dpi = 0  # Select default DPI of 72 by specifying 0.

        # Check if font name/style matches a font loaded into memory by user
        lname = name and name.lower() or ''
        if (lname, bold, italic) in self._memory_fonts:
            font = self._memory_fonts[lname, bold, italic]
            self._set_face(font.face, size, dpi)
            return

        # Use fontconfig to match the font (or substitute a default).
        ft_library = ft_get_library()

        match = self.get_fontconfig_match(name, size, bold, italic)
        if not match:
            raise base.FontException('Could not match font "%s"' % name)

        f = FT_Face()
        if fontconfig.FcPatternGetFTFace(match, FC_FT_FACE, 0, byref(f)) != 0:
            value = FcValue()
            result = fontconfig.FcPatternGet(match, FC_FILE, 0, byref(value))
            if result != 0:
                raise base.FontException('No filename or FT face for "%s"' % \
                                         name)
            result = FT_New_Face(ft_library, value.u.s, 0, byref(f))
            if result:
                raise base.FontException('Could not load "%s": %d' % \
                                         (name, result))

        fontconfig.FcPatternDestroy(match)

        self._set_face(f, size, dpi)

    def _set_face(self, face, size, dpi):
        self.face = face.contents
        self._face_size = float_to_f26p6(size)
        self._dpi = dpi

        FT_Set_Char_Size(self.face, 0, float_to_f26p6(size), dpi, dpi)
        metrics = self.face.size.contents.metrics
        self.ascent = int(f26p6_to_float(metrics.ascender))
        self.descent = int(f26p6_to_float(metrics.descender))

    @staticmethod
    def get_fontconfig_match(name, size, bold, italic):
        if bold:
            bold = FC_WEIGHT_BOLD
        else:
            bold = FC_WEIGHT_REGULAR

        if italic:
            italic = FC_SLANT_ITALIC
        else:
            italic = FC_SLANT_ROMAN

        fontconfig.FcInit()

        pattern = fontconfig.FcPatternCreate()
        fontconfig.FcPatternAddDouble(pattern, FC_SIZE, c_double(size))
        fontconfig.FcPatternAddInteger(pattern, FC_WEIGHT, bold)
        fontconfig.FcPatternAddInteger(pattern, FC_SLANT, italic)
        fontconfig.FcPatternAddString(pattern, FC_FAMILY, name)
        fontconfig.FcConfigSubstitute(0, pattern, FcMatchPattern)
        fontconfig.FcDefaultSubstitute(pattern)

        # Look for a font that matches pattern
        result = FcResult()
        match = fontconfig.FcFontMatch(0, pattern, byref(result))
        fontconfig.FcPatternDestroy(pattern)
        
        return match

    @classmethod
    def have_font(cls, name):
        value = FcValue()
        match = cls.get_fontconfig_match(name, 12, False, False)
        result = fontconfig.FcPatternGet(match, FC_FAMILY, 0, byref(value))
        return value.u.s == name
    
    @classmethod
    def add_font_data(cls, data):
        font = FreeTypeMemoryFont(data)
        cls._memory_fonts[font.name.lower(), font.bold, font.italic] = font
